@@ -833,6 +833,8 @@ def create(vm_):
             )
             for private_ip in private:
                 private_ip = preferred_ip(vm_, [private_ip])
+                if private_ip is False:
+                    continue
                 if salt.utils.cloud.is_public_ip(private_ip):
                     log.warn('{0} is a public IP'.format(private_ip))
                     data.public_ips.append(private_ip)
