@@ -15,9 +15,11 @@ from __future__ import absolute_import
 import copy
 import logging
 import re
+import os.path
 
 # Import salt libs
 import salt.utils
+import salt.utils.itertools
 import salt.utils.systemd
 from salt.exceptions import CommandExecutionError, MinionError
 
@@ -34,9 +36,9 @@ def __virtual__():
     '''
     Set the virtual pkg module if the os is Arch
     '''
-    if __grains__['os'] in ('Arch', 'Arch ARM', 'ManjaroLinux'):
+    if __grains__['os_family'] == 'Arch':
         return __virtualname__
-    return False
+    return (False, 'The pacman module could not be loaded: unsupported OS family.')
 
 
 def _list_removed(old, new):
@@ -75,10 +77,16 @@ def latest_version(*names, **kwargs):
     # Initialize the dict with empty strings
     for name in names:
         ret[name] = ''
-    cmd = 'pacman -Sp --needed --print-format "%n %v" ' \
-          '{0}'.format(' '.join(names))
-    out = __salt__['cmd.run_stdout'](cmd, output_loglevel='trace')
-    for line in out.splitlines():
+    cmd = ['pacman', '-Sp', '--needed', '--print-format', '%n %v']
+    cmd.extend(names)
+
+    if 'root' in kwargs:
+        cmd.extend(('-r', kwargs['root']))
+
+    out = __salt__['cmd.run_stdout'](cmd,
+                                     output_loglevel='trace',
+                                     python_shell=False)
+    for line in salt.utils.itertools.split(out, '\n'):
         try:
             name, version_num = line.split()
             # Only add to return dict if package is in the list of packages
@@ -89,14 +97,6 @@ def latest_version(*names, **kwargs):
         except (ValueError, IndexError):
             pass
 
-    pkgs = {}
-
-    for name in names:
-        if not ret[name]:
-            if not pkgs:
-                pkgs = list_pkgs()
-            if name in pkgs:
-                ret[name] = pkgs[name]
     # Return a string if only one package name passed
     if len(names) == 1:
         return ret[names[0]]
@@ -119,7 +119,7 @@ def upgrade_available(name):
     return latest_version(name) != ''
 
 
-def list_upgrades(refresh=False, **kwargs):  # pylint: disable=W0613
+def list_upgrades(refresh=False, root=None, **kwargs):  # pylint: disable=W0613
     '''
     List all available package upgrades on this system
 
@@ -132,6 +132,9 @@ def list_upgrades(refresh=False, **kwargs):  # pylint: disable=W0613
     upgrades = {}
     cmd = ['pacman', '-S', '-p', '-u', '--print-format', '%n %v']
 
+    if root is not None:
+        cmd.extend(('-r', root))
+
     if refresh:
         cmd.append('-y')
 
@@ -145,17 +148,24 @@ def list_upgrades(refresh=False, **kwargs):  # pylint: disable=W0613
             comment += call['stderr']
         if 'stdout' in call:
             comment += call['stdout']
-        raise CommandExecutionError(
-            '{0}'.format(comment)
-        )
+        if comment:
+            comment = ': ' + comment
+        raise CommandExecutionError('Error listing upgrades' + comment)
     else:
         out = call['stdout']
 
-    for line in out.splitlines():
-        comps = line.split(' ')
-        if len(comps) != 2:
+    for line in salt.utils.itertools.split(out, '\n'):
+        try:
+            pkgname, pkgver = line.split()
+        except ValueError:
             continue
-        upgrades[comps[0]] = comps[1]
+        if pkgname.lower() == 'downloading' and '.db' in pkgver.lower():
+            # Antergos (and possibly other Arch derivatives) add lines when pkg
+            # metadata is being downloaded. Because these lines, when split,
+            # contain two columns (i.e. 'downloading community.db...'), we will
+            # skip this line to keep it from being interpreted as an upgrade.
+            continue
+        upgrades[pkgname] = pkgver
     return upgrades
 
 
@@ -201,10 +211,14 @@ def list_pkgs(versions_as_list=False, **kwargs):
             __salt__['pkg_resource.stringify'](ret)
             return ret
 
-    cmd = 'pacman -Q'
+    cmd = ['pacman', '-Q']
+
+    if 'root' in kwargs:
+        cmd.extend(('-r', kwargs['root']))
+
     ret = {}
-    out = __salt__['cmd.run'](cmd, output_loglevel='trace')
-    for line in out.splitlines():
+    out = __salt__['cmd.run'](cmd, output_loglevel='trace', python_shell=False)
+    for line in salt.utils.itertools.split(out, '\n'):
         if not line:
             continue
         try:
@@ -222,7 +236,7 @@ def list_pkgs(versions_as_list=False, **kwargs):
     return ret
 
 
-def refresh_db():
+def refresh_db(root=None):
     '''
     Just run a ``pacman -Sy``, return a dict::
 
@@ -234,22 +248,27 @@ def refresh_db():
 
         salt '*' pkg.refresh_db
     '''
-    cmd = 'LANG=C pacman -Sy'
+    cmd = ['pacman', '-Sy']
+
+    if root is not None:
+        cmd.extend(('-r', root))
+
     ret = {}
-    call = __salt__['cmd.run_all'](cmd, output_loglevel='trace',
-            python_shell=True)
+    call = __salt__['cmd.run_all'](cmd,
+                                   output_loglevel='trace',
+                                   env={'LANG': 'C'},
+                                   python_shell=False)
     if call['retcode'] != 0:
         comment = ''
         if 'stderr' in call:
-            comment += call['stderr']
-
+            comment += ': ' + call['stderr']
         raise CommandExecutionError(
-            '{0}'.format(comment)
+            'Error refreshing package database' + comment
         )
     else:
         out = call['stdout']
 
-    for line in out.splitlines():
+    for line in salt.utils.itertools.split(out, '\n'):
         if line.strip().startswith('::'):
             continue
         if not line:
@@ -270,7 +289,7 @@ def install(name=None,
             sources=None,
             **kwargs):
     '''
-    .. versionchanged:: 2015.8.12,2016.3.3,Carbon
+    .. versionchanged:: 2015.8.12,2016.3.3,2016.11.0
         On minions running systemd>=205, `systemd-run(1)`_ is now used to
         isolate commands which modify installed packages from the
         ``salt-minion`` daemon's control group. This is done to keep systemd
@@ -356,12 +375,15 @@ def install(name=None,
     version_num = kwargs.get('version')
     if version_num:
         if pkgs is None and sources is None:
-            # Allow "version" to work for single package target
+            # Allow 'version' to work for single package target
             pkg_params = {name: version_num}
         else:
             log.warning('\'version\' parameter will be ignored for multiple '
                         'package targets')
 
+    if 'root' in kwargs:
+        pkg_params['-r'] = kwargs['root']
+
     cmd = []
     if salt.utils.systemd.has_scope(__context__) \
             and __salt__['config.get']('systemd.scope', True):
@@ -404,15 +426,33 @@ def install(name=None,
         cmd.extend(targets)
 
     old = list_pkgs()
-    __salt__['cmd.run'](cmd, output_loglevel='trace', python_shell=False)
+    out = __salt__['cmd.run_all'](
+        cmd,
+        output_loglevel='trace',
+        python_shell=False
+    )
+
+    if out['retcode'] != 0 and out['stderr']:
+        errors = [out['stderr']]
+    else:
+        errors = []
+
     __context__.pop('pkg.list_pkgs', None)
     new = list_pkgs()
-    return salt.utils.compare_dicts(old, new)
+    ret = salt.utils.compare_dicts(old, new)
+
+    if errors:
+        raise CommandExecutionError(
+            'Problem encountered installing package(s)',
+            info={'errors': errors, 'changes': ret}
+        )
+
+    return ret
 
 
-def upgrade(refresh=False, **kwargs):
+def upgrade(refresh=False, root=None, **kwargs):
     '''
-    .. versionchanged:: 2015.8.12,2016.3.3,Carbon
+    .. versionchanged:: 2015.8.12,2016.3.3,2016.11.0
         On minions running systemd>=205, `systemd-run(1)`_ is now used to
         isolate commands which modify installed packages from the
         ``salt-minion`` daemon's control group. This is done to keep systemd
@@ -444,8 +484,7 @@ def upgrade(refresh=False, **kwargs):
     '''
     ret = {'changes': {},
            'result': True,
-           'comment': '',
-           }
+           'comment': ''}
 
     old = list_pkgs()
 
@@ -457,19 +496,23 @@ def upgrade(refresh=False, **kwargs):
     if salt.utils.is_true(refresh):
         cmd.append('-y')
 
+    if root is not None:
+        cmd.extend(('-r', root))
+
     call = __salt__['cmd.run_all'](cmd,
                                    output_loglevel='trace',
-                                   python_shell=False)
+                                   python_shell=False,
+                                   redirect_stderr=True)
+
     if call['retcode'] != 0:
         ret['result'] = False
-        if 'stderr' in call:
-            ret['comment'] += call['stderr']
-        if 'stdout' in call:
-            ret['comment'] += call['stdout']
-    else:
-        __context__.pop('pkg.list_pkgs', None)
-        new = list_pkgs()
-        ret['changes'] = salt.utils.compare_dicts(old, new)
+        if call['stdout']:
+            ret['comment'] = call['stdout']
+
+    __context__.pop('pkg.list_pkgs', None)
+    new = list_pkgs()
+    ret['changes'] = salt.utils.compare_dicts(old, new)
+
     return ret
 
 
@@ -487,6 +530,7 @@ def _uninstall(action='remove', name=None, pkgs=None, **kwargs):
     targets = [x for x in pkg_params if x in old]
     if not targets:
         return {}
+
     remove_arg = '-Rsc' if action == 'purge' else '-R'
 
     cmd = []
@@ -496,15 +540,36 @@ def _uninstall(action='remove', name=None, pkgs=None, **kwargs):
     cmd.extend(['pacman', remove_arg, '--noprogressbar', '--noconfirm'])
     cmd.extend(targets)
 
-    __salt__['cmd.run'](cmd, output_loglevel='trace', python_shell=False)
+    if 'root' in kwargs:
+        cmd.extend(('-r', kwargs['root']))
+
+    out = __salt__['cmd.run_all'](
+        cmd,
+        output_loglevel='trace',
+        python_shell=False
+    )
+
+    if out['retcode'] != 0 and out['stderr']:
+        errors = [out['stderr']]
+    else:
+        errors = []
+
     __context__.pop('pkg.list_pkgs', None)
     new = list_pkgs()
-    return salt.utils.compare_dicts(old, new)
+    ret = salt.utils.compare_dicts(old, new)
+
+    if errors:
+        raise CommandExecutionError(
+            'Problem encountered removing package(s)',
+            info={'errors': errors, 'changes': ret}
+        )
+
+    return ret
 
 
 def remove(name=None, pkgs=None, **kwargs):
     '''
-    .. versionchanged:: 2015.8.12,2016.3.3,Carbon
+    .. versionchanged:: 2015.8.12,2016.3.3,2016.11.0
         On minions running systemd>=205, `systemd-run(1)`_ is now used to
         isolate commands which modify installed packages from the
         ``salt-minion`` daemon's control group. This is done to keep systemd
@@ -548,7 +613,7 @@ def remove(name=None, pkgs=None, **kwargs):
 
 def purge(name=None, pkgs=None, **kwargs):
     '''
-    .. versionchanged:: 2015.8.12,2016.3.3,Carbon
+    .. versionchanged:: 2015.8.12,2016.3.3,2016.11.0
         On minions running systemd>=205, `systemd-run(1)`_ is now used to
         isolate commands which modify installed packages from the
         ``salt-minion`` daemon's control group. This is done to keep systemd
@@ -607,9 +672,16 @@ def file_list(*packages):
     '''
     errors = []
     ret = []
-    cmd = 'pacman -Ql {0}'.format(' '.join(packages))
-    out = __salt__['cmd.run'](cmd, output_loglevel='trace')
-    for line in out.splitlines():
+    cmd = ['pacman', '-Ql']
+
+    if len(packages) > 0 and os.path.exists(packages[0]):
+        packages = list(packages)
+        cmd.extend(('-r', packages.pop(0)))
+
+    cmd.extend(packages)
+
+    out = __salt__['cmd.run'](cmd, output_loglevel='trace', python_shell=False)
+    for line in salt.utils.itertools.split(out, '\n'):
         if line.startswith('error'):
             errors.append(line)
         else:
@@ -634,9 +706,16 @@ def file_dict(*packages):
     '''
     errors = []
     ret = {}
-    cmd = 'pacman -Ql {0}'.format(' '.join(packages))
-    out = __salt__['cmd.run'](cmd, output_loglevel='trace')
-    for line in out.splitlines():
+    cmd = ['pacman', '-Ql']
+
+    if len(packages) > 0 and os.path.exists(packages[0]):
+        packages = list(packages)
+        cmd.extend(('-r', packages.pop(0)))
+
+    cmd.extend(packages)
+
+    out = __salt__['cmd.run'](cmd, output_loglevel='trace', python_shell=False)
+    for line in salt.utils.itertools.split(out, '\n'):
         if line.startswith('error'):
             errors.append(line)
         else:
@@ -667,9 +746,11 @@ def owner(*paths):
     if not paths:
         return ''
     ret = {}
-    cmd = 'pacman -Qqo {0!r}'
+    cmd_prefix = ['pacman', '-Qqo']
+
     for path in paths:
-        ret[path] = __salt__['cmd.run_stdout'](cmd.format(path))
+        ret[path] = __salt__['cmd.run_stdout'](cmd_prefix + [path],
+                                               python_shell=False)
     if len(ret) == 1:
         return next(six.itervalues(ret))
     return ret
