@@ -3,41 +3,43 @@
 Execute calls on selinux
 
 .. note::
-    This module requires the ``semanage`` and ``setsebool`` commands to be
-    available on the minion. On RHEL-based distros, this means that the
-    ``policycoreutils`` and ``policycoreutils-python`` packages must be
-    installed. If not on a RHEL-based distribution, consult the selinux
-    documentation for your distro to ensure that the proper packages are
-    installed.
+    This module requires the ``semanage``, ``setsebool``, and ``semodule``
+    commands to be available on the minion. On RHEL-based distributions,
+    ensure that the ``policycoreutils`` and ``policycoreutils-python``
+    packages are installed. If not on a Fedora or RHEL-based distribution,
+    consult the selinux documentation for your distribution to ensure that the
+    proper packages are installed.
 '''
-from __future__ import absolute_import
 
 # Import python libs
+from __future__ import absolute_import
 import os
 
 # Import salt libs
 import salt.utils
 import salt.utils.decorators as decorators
-from salt.ext.six import string_types
 from salt.exceptions import CommandExecutionError
 
+# Import 3rd-party libs
+import salt.ext.six as six
+
 
 def __virtual__():
     '''
     Check if the os is Linux, and then if selinux is running in permissive or
     enforcing mode.
     '''
-    required_cmds = ('semanage', 'setsebool')
+    required_cmds = ('semanage', 'setsebool', 'semodule')
 
     # Iterate over all of the commands this module uses and make sure
     # each of them are available in the standard PATH to prevent breakage
     for cmd in required_cmds:
         if not salt.utils.which(cmd):
-            return False
+            return (False, cmd + ' is not in the path')
     # SELinux only makes sense on Linux *obviously*
     if __grains__['kernel'] == 'Linux' and selinux_fs_path():
         return 'selinux'
-    return False
+    return (False, 'Module only works on Linux with selinux installed')
 
 
 # Cache the SELinux directory to not look it up over and over
@@ -93,7 +95,7 @@ def setenforce(mode):
 
         salt '*' selinux.setenforce enforcing
     '''
-    if isinstance(mode, string_types):
+    if isinstance(mode, six.string_types):
         if mode.lower() == 'enforcing':
             mode = '1'
         elif mode.lower() == 'permissive':
@@ -163,7 +165,7 @@ def setsebools(pairs, persist=False):
         cmd = 'setsebool -P '
     else:
         cmd = 'setsebool '
-    for boolean, value in pairs.items():
+    for boolean, value in six.iteritems(pairs):
         cmd = '{0} {1}={2}'.format(cmd, boolean, value)
     return not __salt__['cmd.retcode'](cmd, python_shell=False)
 
@@ -189,3 +191,85 @@ def list_sebool():
                          'Default': comps[3][:-1],
                          'Description': ' '.join(comps[4:])}
     return ret
+
+
+def getsemod(module):
+    '''
+    Return the information on a specific selinux module
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' selinux.getsemod mysql
+
+    .. versionadded:: 2016.3.0
+    '''
+    return list_semod().get(module, {})
+
+
+def setsemod(module, state):
+    '''
+    Enable or disable an SELinux module.
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' selinux.setsemod nagios Enabled
+
+    .. versionadded:: 2016.3.0
+    '''
+    if state.lower() == 'enabled':
+        cmd = 'semodule -e {0}'.format(module)
+    elif state.lower() == 'disabled':
+        cmd = 'semodule -d {0}'.format(module)
+    return not __salt__['cmd.retcode'](cmd)
+
+
+def list_semod():
+    '''
+    Return a structure listing all of the selinux modules on the system and
+    what state they are in
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' selinux.list_semod
+
+    .. versionadded:: 2016.3.0
+    '''
+    helptext = __salt__['cmd.run']('semodule -h').splitlines()
+    semodule_version = ''
+    for line in helptext:
+        if line.strip().startswith('full'):
+            semodule_version = 'new'
+
+    if semodule_version == 'new':
+        mdata = __salt__['cmd.run']('semodule -lfull').splitlines()
+        ret = {}
+        for line in mdata:
+            if not line.strip():
+                continue
+            comps = line.split()
+            if len(comps) == 4:
+                ret[comps[1]] = {'Enabled': False,
+                                 'Version': None}
+            else:
+                ret[comps[1]] = {'Enabled': True,
+                                 'Version': None}
+    else:
+        mdata = __salt__['cmd.run']('semodule -l').splitlines()
+        ret = {}
+        for line in mdata:
+            if not line.strip():
+                continue
+            comps = line.split()
+            if len(comps) == 3:
+                ret[comps[0]] = {'Enabled': False,
+                                 'Version': comps[1]}
+            else:
+                ret[comps[0]] = {'Enabled': True,
+                                 'Version': comps[1]}
+    return ret
