@@ -2,6 +2,12 @@
 '''
 Module for managing Windows Users
 
+.. important::
+    If you feel that Salt should be using this module to manage users on a
+    minion, and it is using a different module (or gives an error similar to
+    *'user.info' is not available*), see :ref:`here
+    <module-provider-override>`.
+
 :depends:
         - pywintypes
         - win32api
@@ -11,9 +17,12 @@ Module for managing Windows Users
         - win32security
         - win32ts
 
-NOTE: This currently only works with local user accounts, not domain accounts
+.. note::
+    This currently only works with local user accounts, not domain accounts
 '''
 from __future__ import absolute_import
+from datetime import datetime
+import time
 
 try:
     from shlex import quote as _cmd_quote  # pylint: disable=E0611
@@ -30,7 +39,10 @@ log = logging.getLogger(__name__)
 
 try:
     import pywintypes
+    import wmi
+    import pythoncom
     import win32api
+    import win32con
     import win32net
     import win32netcon
     import win32profile
@@ -50,7 +62,7 @@ def __virtual__():
     '''
     if HAS_WIN32NET_MODS and salt.utils.is_windows():
         return __virtualname__
-    return False
+    return (False, "Module win_useradd: module has failed dependencies or is not on Windows client")
 
 
 def add(name,
@@ -65,37 +77,38 @@ def add(name,
     '''
     Add a user to the minion.
 
-    :param name: str
-    User name
+    :param str name:
+        User name
 
-    :param password: str
-    User's password in plain text.
+    :param str password:
+        User's password in plain text.
 
-    :param fullname: str
-    The user's full name.
+    :param str fullname:
+        The user's full name.
 
-    :param description: str
-    A brief description of the user account.
+    :param str description:
+        A brief description of the user account.
 
-    :param groups: list
-    A list of groups to add the user to.
+    :param list groups:
+        A list of groups to add the user to.
 
-    :param home: str
-    The path to the user's home directory.
+    :param str home:
+        The path to the user's home directory.
 
-    :param homedrive: str
-    The drive letter to assign to the home directory. Must be the Drive Letter
-    followed by a colon. ie: U:
+    :param str homedrive:
+        The drive letter to assign to the home directory. Must be the Drive Letter
+        followed by a colon. ie: U:
 
-    :param profile: str
-    An explicit path to a profile. Can be a UNC or a folder on the system. If
-    left blank, windows uses it's default profile directory.
+    :param str profile:
+        An explicit path to a profile. Can be a UNC or a folder on the system. If
+        left blank, windows uses it's default profile directory.
 
-    :param logonscript: str
-    Path to a login script to run when the user logs on.
+    :param str logonscript:
+        Path to a login script to run when the user logs on.
 
-    :return: bool
-    True if successful. False is unsuccessful.
+    :return:
+        True if successful. False is unsuccessful.
+    :rtype: bool
 
     CLI Example:
 
@@ -142,40 +155,68 @@ def update(name,
            home=None,
            homedrive=None,
            logonscript=None,
-           profile=None):
+           profile=None,
+           expiration_date=None,
+           expired=None,
+           account_disabled=None,
+           unlock_account=None,
+           password_never_expires=None,
+           disallow_change_password=None):
     r'''
     Updates settings for the windows user. Name is the only required parameter.
     Settings will only be changed if the parameter is passed a value.
 
     .. versionadded:: 2015.8.0
 
-    :param name: str
-    The user name to update.
+    :param str name:
+        The user name to update.
 
-    :param password: str
-    New user password in plain text.
+    :param str password:
+        New user password in plain text.
 
-    :param fullname: str
-    The user's full name.
+    :param str fullname:
+        The user's full name.
 
-    :param description: str
-    A brief description of the user account.
+    :param str description:
+        A brief description of the user account.
 
-    :param home: str
-    The path to the user's home directory.
+    :param str home:
+        The path to the user's home directory.
 
-    :param homedrive: str
-    The drive letter to assign to the home directory. Must be the Drive Letter
-    followed by a colon. ie: U:
+    :param str homedrive:
+        The drive letter to assign to the home directory. Must be the Drive Letter
+        followed by a colon. ie: U:
 
-    :param logonscript: str
-    The path to the logon script.
+    :param str logonscript:
+        The path to the logon script.
 
-    :param profile: str
-    The path to the user's profile directory.
+    :param str profile:
+        The path to the user's profile directory.
 
-    :return: bool
-    True if successful. False is unsuccessful.
+    :param date expiration_date: The date and time when the account expires. Can
+        be a valid date/time string. To set to never expire pass the string 'Never'.
+
+    :param bool expired: Pass `True` to expire the account. The user will be
+        prompted to change their password at the next logon. Pass `False` to mark
+        the account as 'not expired'. You can't use this to negate the expiration if
+        the expiration was caused by the account expiring. You'll have to change
+        the `expiration_date` as well.
+
+    :param bool account_disabled: True disables the account. False enables the
+        account.
+
+    :param bool unlock_account: True unlocks a locked user account. False is
+        ignored.
+
+    :param bool password_never_expires: True sets the password to never expire.
+        False allows the password to expire.
+
+    :param bool disallow_change_password: True blocks the user from changing
+        the password. False allows the user to change the password.
+
+    :return: True if successful. False is unsuccessful.
+
+    :rtype: bool
 
     CLI Example:
 
@@ -213,6 +254,38 @@ def update(name,
         user_info['full_name'] = fullname
     if profile:
         user_info['profile'] = profile
+    if expiration_date:
+        if expiration_date == 'Never':
+            user_info['acct_expires'] = win32netcon.TIMEQ_FOREVER
+        else:
+            try:
+                dt_obj = salt.utils.date_cast(expiration_date)
+            except (ValueError, RuntimeError):
+                return 'Invalid Date/Time Format: {0}'.format(expiration_date)
+            user_info['acct_expires'] = time.mktime(dt_obj.timetuple())
+    if expired is not None:
+        if expired:
+            user_info['password_expired'] = 1
+        else:
+            user_info['password_expired'] = 0
+    if account_disabled is not None:
+        if account_disabled:
+            user_info['flags'] |= win32netcon.UF_ACCOUNTDISABLE
+        else:
+            user_info['flags'] ^= win32netcon.UF_ACCOUNTDISABLE
+    if unlock_account is not None:
+        if unlock_account:
+            user_info['flags'] ^= win32netcon.UF_LOCKOUT
+    if password_never_expires is not None:
+        if password_never_expires:
+            user_info['flags'] |= win32netcon.UF_DONT_EXPIRE_PASSWD
+        else:
+            user_info['flags'] ^= win32netcon.UF_DONT_EXPIRE_PASSWD
+    if disallow_change_password is not None:
+        if disallow_change_password:
+            user_info['flags'] |= win32netcon.UF_PASSWD_CANT_CHANGE
+        else:
+            user_info['flags'] ^= win32netcon.UF_PASSWD_CANT_CHANGE
 
     # Apply new settings
     try:
@@ -234,16 +307,20 @@ def delete(name,
     '''
     Remove a user from the minion
 
-    :param name:
-    The name of the user to delete
+    :param str name:
+        The name of the user to delete
 
-    :param purge:
-    Boolean value indicating that the user profile should also be removed when
-    the user account is deleted. If set to True the profile will be removed.
+    :param bool purge:
+        Boolean value indicating that the user profile should also be removed when
+        the user account is deleted. If set to True the profile will be removed.
 
-    :param force:
-    Boolean value indicating that the user account should be deleted even if the
-    user is logged in. True will log the user out and delete user.
+    :param bool force:
+        Boolean value indicating that the user account should be deleted even if the
+        user is logged in. True will log the user out and delete user.
+
+    :return:
+        True if successful
+    :rtype: bool
 
     CLI Example:
 
@@ -329,6 +406,22 @@ def delete(name,
 
 
 def getUserSid(username):
+    '''
+    Get the Security ID for the user
+
+    :param str username:
+        user name for which to look up the SID
+
+    :return:
+        Returns the user SID
+    :rtype: str
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' user.getUserSid jsnuffy
+    '''
     domain = win32api.GetComputerName()
     if username.find(u'\\') != -1:
         domain = username.split(u'\\')[0]
@@ -339,13 +432,23 @@ def getUserSid(username):
 
 def setpassword(name, password):
     '''
-    Set a user's password
+    Set the user's password
+
+    :param str name:
+        user name for which to set the password
+
+    :param str password:
+        the new password
+
+    :return:
+        True if successful. False is unsuccessful.
+    :rtype: bool
 
     CLI Example:
 
     .. code-block:: bash
 
-        salt '*' user.setpassword name password
+        salt '*' user.setpassword jsnuffy sup3rs3cr3t
     '''
     return update(name=name, password=password)
 
@@ -354,11 +457,21 @@ def addgroup(name, group):
     '''
     Add user to a group
 
+    :param str name:
+        user name to add to the group
+
+    :param str group:
+        name of the group to which to add the user
+
+    :return:
+        True if successful. False is unsuccessful.
+    :rtype: bool
+
     CLI Example:
 
     .. code-block:: bash
 
-        salt '*' user.addgroup username groupname
+        salt '*' user.addgroup jsnuffy 'Power Users'
     '''
     name = _cmd_quote(name)
     group = _cmd_quote(group).lstrip('\'').rstrip('\'')
@@ -379,11 +492,21 @@ def removegroup(name, group):
     '''
     Remove user from a group
 
+    :param str name:
+        user name to remove from the group
+
+    :param str group:
+        name of the group from which to remove the user
+
+    :return:
+        True if successful. False is unsuccessful.
+    :rtype: bool
+
     CLI Example:
 
     .. code-block:: bash
 
-        salt '*' user.removegroup username groupname
+        salt '*' user.removegroup jsnuffy 'Power Users'
     '''
     name = _cmd_quote(name)
     group = _cmd_quote(group).lstrip('\'').rstrip('\'')
@@ -407,6 +530,19 @@ def chhome(name, home, persist=False):
     Change the home directory of the user, pass True for persist to move files
     to the new home directory if the old home directory exist.
 
+    :param str name:
+        name of the user whose home directory you wish to change
+
+    :param str home:
+        new location of the home directory
+
+    :param bool persist:
+        True to move the contents of the existing home directory to the new location
+
+    :return:
+        True if successful. False is unsuccessful.
+    :rtype: bool
+
     CLI Example:
 
     .. code-block:: bash
@@ -440,6 +576,16 @@ def chprofile(name, profile):
     '''
     Change the profile directory of the user
 
+    :param str name:
+        name of the user whose profile you wish to change
+
+    :param str profile:
+        new location of the profile
+
+    :return: True if successful. False is unsuccessful.
+
+    :rtype: bool
+
     CLI Example:
 
     .. code-block:: bash
@@ -453,6 +599,16 @@ def chfullname(name, fullname):
     '''
     Change the full name of the user
 
+    :param str name:
+        user name for which to change the full name
+
+    :param str fullname:
+        the new value for the full name
+
+    :return:
+        True if successful. False is unsuccessful.
+    :rtype: bool
+
     CLI Example:
 
     .. code-block:: bash
@@ -467,11 +623,26 @@ def chgroups(name, groups, append=True):
     Change the groups this user belongs to, add append=False to make the user a
     member of only the specified groups
 
+    :param str name:
+        user name for which to change groups
+
+    :param groups:
+        a single group or a list of groups to assign to the user
+    :type groups: list, str
+
+    :param bool append:
+        True adds the passed groups to the user's current groups
+        False sets the user's groups to the passed groups only
+
+    :return:
+        True if successful. False is unsuccessful.
+    :rtype: bool
+
     CLI Example:
 
     .. code-block:: bash
 
-        salt '*' user.chgroups foo wheel,root True
+        salt '*' user.chgroups jsnuffy Administrators,Users True
     '''
     if isinstance(groups, string_types):
         groups = groups.split(',')
@@ -495,7 +666,10 @@ def chgroups(name, groups, append=True):
             continue
         group = _cmd_quote(group).lstrip('\'').rstrip('\'')
         cmd = 'net localgroup "{0}" {1} /add'.format(group, name)
-        __salt__['cmd.run_all'](cmd, python_shell=True)
+        out = __salt__['cmd.run_all'](cmd, python_shell=True)
+        if out['retcode'] != 0:
+            log.error(out['stdout'])
+            return False
 
     agrps = set(list_groups(name))
     return len(ugrps - agrps) == 0
@@ -505,30 +679,39 @@ def info(name):
     '''
     Return user information
 
-    :param name: str
-    Username for which to display information
+    :param str name:
+        Username for which to display information
 
-    :returns: dict
-    A dictionary containing user information
-    - fullname
-    - username
-    - uid
-    - passwd (will always return None)
-    - comment (same as description, left here for backwards compatibility)
-    - description
-    - active
-    - logonscript
-    - profile
-    - home
-    - homedrive
-    - groups
-    - gid
+    :returns:
+        A dictionary containing user information
+            - fullname
+            - username
+            - SID
+            - passwd (will always return None)
+            - comment (same as description, left here for backwards compatibility)
+            - description
+            - active
+            - logonscript
+            - profile
+            - home
+            - homedrive
+            - groups
+            - password_changed
+            - successful_logon_attempts
+            - failed_logon_attempts
+            - last_logon
+            - account_disabled
+            - account_locked
+            - password_never_expires
+            - disallow_change_password
+            - gid
+    :rtype: dict
 
     CLI Example:
 
     .. code-block:: bash
 
-        salt '*' user.info root
+        salt '*' user.info jsnuffy
     '''
     ret = {}
     items = {}
@@ -553,6 +736,19 @@ def info(name):
         ret['active'] = (not bool(items['flags'] & win32netcon.UF_ACCOUNTDISABLE))
         ret['logonscript'] = items['script_path']
         ret['profile'] = items['profile']
+        ret['failed_logon_attempts'] = items['bad_pw_count']
+        ret['successful_logon_attempts'] = items['num_logons']
+        secs = time.mktime(datetime.now().timetuple()) - items['password_age']
+        ret['password_changed'] = datetime.fromtimestamp(secs). \
+            strftime('%Y-%m-%d %H:%M:%S')
+        if items['last_logon'] == 0:
+            ret['last_logon'] = 'Never'
+        else:
+            ret['last_logon'] = datetime.fromtimestamp(items['last_logon']).\
+                strftime('%Y-%m-%d %H:%M:%S')
+        ret['expiration_date'] = datetime.fromtimestamp(items['acct_expires']).\
+            strftime('%Y-%m-%d %H:%M:%S')
+        ret['expired'] = items['password_expired'] == 1
         if not ret['profile']:
             ret['profile'] = _get_userprofile_from_registry(name, ret['uid'])
         ret['home'] = items['home_dir']
@@ -560,9 +756,30 @@ def info(name):
         if not ret['home']:
             ret['home'] = ret['profile']
         ret['groups'] = groups
+        if items['flags'] & win32netcon.UF_DONT_EXPIRE_PASSWD == 0:
+            ret['password_never_expires'] = False
+        else:
+            ret['password_never_expires'] = True
+        if items['flags'] & win32netcon.UF_ACCOUNTDISABLE == 0:
+            ret['account_disabled'] = False
+        else:
+            ret['account_disabled'] = True
+        if items['flags'] & win32netcon.UF_LOCKOUT == 0:
+            ret['account_locked'] = False
+        else:
+            ret['account_locked'] = True
+        if items['flags'] & win32netcon.UF_PASSWD_CANT_CHANGE == 0:
+            ret['disallow_change_password'] = False
+        else:
+            ret['disallow_change_password'] = True
+
         ret['gid'] = ''
 
-    return ret
+        return ret
+
+    else:
+
+        return False
 
 
 def _get_userprofile_from_registry(user, sid):
@@ -570,10 +787,11 @@ def _get_userprofile_from_registry(user, sid):
     In case net user doesn't return the userprofile
     we can get it from the registry
     '''
-    profile_dir = __salt__['reg.read_key'](
-        'HKEY_LOCAL_MACHINE', u'SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\ProfileList\\{0}'.format(sid),
+    profile_dir = __salt__['reg.read_value'](
+        'HKEY_LOCAL_MACHINE',
+        u'SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\ProfileList\\{0}'.format(sid),
         'ProfileImagePath'
-    )
+    )['vdata']
     log.debug(u'user {0} with sid={2} profile is located at "{1}"'.format(user, profile_dir, sid))
     return profile_dir
 
@@ -582,6 +800,13 @@ def list_groups(name):
     '''
     Return a list of groups the named user belongs to
 
+    :param str name:
+        user name for which to list groups
+
+    :return:
+        list of groups to which the user belongs
+    :rtype: list
+
     CLI Example:
 
     .. code-block:: bash
@@ -603,6 +828,14 @@ def getent(refresh=False):
     '''
     Return the list of all info for all users
 
+    :param bool refresh:
+        Refresh the cached user information. Default is False. Useful when used from
+        within a state function.
+
+    :return:
+        A dictionary containing information about all users on the system
+    :rtype: dict
+
     CLI Example:
 
     .. code-block:: bash
@@ -634,6 +867,16 @@ def getent(refresh=False):
 def list_users():
     '''
     Return a list of users on Windows
+
+    :return:
+        list of users on the system
+    :rtype: list
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' user.list_users
     '''
     res = 0
     users = []
@@ -660,21 +903,110 @@ def rename(name, new_name):
     '''
     Change the username for a named user
 
+    :param str name:
+        user name to change
+
+    :param str new_name:
+        the new name for the current user
+
+    :return:
+        True if successful. False is unsuccessful.
+    :rtype: bool
+
     CLI Example:
 
     .. code-block:: bash
 
-        salt '*' user.rename name new_name
+        salt '*' user.rename jsnuffy jshmoe
     '''
+    # Load information for the current name
     current_info = info(name)
     if not current_info:
-        raise CommandExecutionError('User {0!r} does not exist'.format(name))
+        raise CommandExecutionError('User \'{0}\' does not exist'.format(name))
+
+    # Look for an existing user with the new name
     new_info = info(new_name)
     if new_info:
-        raise CommandExecutionError('User {0!r} already exists'.format(new_name))
-    cmd = 'wmic useraccount where name="{0}" rename {1}'.format(name, new_name)
-    __salt__['cmd.run'](cmd)
-    post_info = info(new_name)
-    if post_info['name'] != current_info['name']:
-        return post_info['name'] == new_name
-    return False
+        raise CommandExecutionError(
+            'User \'{0}\' already exists'.format(new_name)
+        )
+
+    # Rename the user account
+    # Connect to WMI
+    pythoncom.CoInitialize()
+    c = wmi.WMI(find_classes=0)
+
+    # Get the user object
+    try:
+        user = c.Win32_UserAccount(Name=name)[0]
+    except IndexError:
+        raise CommandExecutionError('User \'{0}\' does not exist'.format(name))
+
+    # Rename the user
+    result = user.Rename(new_name)[0]
+
+    # Check the result (0 means success)
+    if not result == 0:
+        # Define Error Dict
+        error_dict = {0: 'Success',
+                      1: 'Instance not found',
+                      2: 'Instance required',
+                      3: 'Invalid parameter',
+                      4: 'User not found',
+                      5: 'Domain not found',
+                      6: 'Operation is allowed only on the primary domain controller of the domain',
+                      7: 'Operation is not allowed on the last administrative account',
+                      8: 'Operation is not allowed on specified special groups: user, admin, local, or guest',
+                      9: 'Other API error',
+                      10: 'Internal error'}
+        raise CommandExecutionError(
+            'There was an error renaming \'{0}\' to \'{1}\'. Error: {2}'
+            .format(name, new_name, error_dict[result])
+        )
+
+    return info(new_name).get('name') == new_name
+
+
+def current(sam=False):
+    '''
+    Get the username that salt-minion is running under. If salt-minion is
+    running as a service it should return the Local System account. If salt is
+    running from a command prompt it should return the username that started the
+    command prompt.
+
+    .. versionadded:: 2015.5.6
+
+    :param bool sam:
+        False returns just the username without any domain notation. True
+        returns the domain with the username in the SAM format. Ie:
+
+        ``domain\\username``
+
+    :return:
+        Returns False if the username cannot be returned. Otherwise returns the
+        username.
+    :rtype: bool str
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' user.current
+    '''
+    try:
+        if sam:
+            user_name = win32api.GetUserNameEx(win32con.NameSamCompatible)
+        else:
+            user_name = win32api.GetUserName()
+    except pywintypes.error as exc:
+        (number, context, message) = exc
+        log.error('Failed to get current user')
+        log.error('nbr: {0}'.format(number))
+        log.error('ctx: {0}'.format(context))
+        log.error('msg: {0}'.format(message))
+        return False
+
+    if not user_name:
+        return False
+
+    return user_name
