@@ -19,33 +19,46 @@ A REST API for Salt
 :client_libraries:
     - Java: https://github.com/SUSE/saltstack-netapi-client-java
     - Python: https://github.com/saltstack/pepper
+:setup:
+    All steps below are performed on the machine running the Salt Master
+    daemon. Configuration goes into the Master configuration file.
+
+    1.  Install ``salt-api``. (This step varies between OS and Linux distros.
+        Some package systems have a split package, others include salt-api in
+        the main Salt package. Ensure the ``salt-api --version`` output matches
+        the ``salt --version`` output.)
+    2.  Install CherryPy. (Read the version caveat in the section above.)
+    3.  Optional: generate self-signed SSL certificates.
+
+        Using a secure HTTPS connection is strongly recommended since Salt
+        eauth authentication credentials will be sent over the wire.
+
+        1.  Install the PyOpenSSL package.
+        2.  Generate a self-signed certificate using the
+            :py:func:`~salt.modules.tls.create_self_signed_cert` execution
+            function.
+
+            .. code-block:: bash
+
+                salt-call --local tls.create_self_signed_cert
+
+    4.  Edit the master config to create at least one external auth user or
+        group following the :ref:`full external auth instructions <acl-eauth>`.
+    5.  Edit the master config with the following production-ready example to
+        enable the ``rest_cherrypy`` module. (Adjust cert paths as needed, or
+        disable SSL (not recommended!).)
+
+        .. code-block:: yaml
+
+            rest_cherrypy:
+              port: 8000
+              ssl_crt: /etc/pki/tls/certs/localhost.crt
+              ssl_key: /etc/pki/tls/certs/localhost.key
+
+    6.  Restart the ``salt-master`` daemon.
+    7.  Start the ``salt-api`` daemon.
+
 :configuration:
-    All authentication is done through Salt's :ref:`external auth
-    <acl-eauth>` system which requires additional configuration not described
-    here.
-
-    Example production-ready configuration; add to the Salt master config file
-    and restart the ``salt-master`` and ``salt-api`` daemons:
-
-    .. code-block:: yaml
-
-        rest_cherrypy:
-          port: 8000
-          ssl_crt: /etc/pki/tls/certs/localhost.crt
-          ssl_key: /etc/pki/tls/certs/localhost.key
-
-    Using only a secure HTTPS connection is strongly recommended since Salt
-    authentication credentials will be sent over the wire.
-
-    A self-signed certificate can be generated using the
-    :py:func:`~salt.modules.tls.create_self_signed_cert` execution function.
-    Running this function requires pyOpenSSL and the ``salt-call`` script is
-    available in the ``salt-minion`` package.
-
-    .. code-block:: bash
-
-        salt-call --local tls.create_self_signed_cert
-
     All available configuration options are detailed below. These settings
     configure the CherryPy HTTP server and do not apply when using an external
     server such as Apache or Nginx.
@@ -112,7 +125,8 @@ Authentication
 Authentication is performed by passing a session token with each request.
 Tokens are generated via the :py:class:`Login` URL.
 
-The token may be sent in one of two ways:
+The token may be sent in one of two ways: as a custom header or as a session
+cookie. The latter is far more convenient for clients that support cookies.
 
 * Include a custom header named :mailheader:`X-Auth-Token`.
 
@@ -165,54 +179,204 @@ The token may be sent in one of two ways:
 Usage
 -----
 
-Commands are sent to a running Salt master via this module by sending HTTP
-requests to the URLs detailed below.
+This interface directly exposes Salt's :ref:`Python API <python-api>`.
+Everything possible at the CLI is possible through the Python API. Commands are
+executed on the Salt Master.
 
-.. admonition:: Content negotiation
+The root URL (``/``) is RPC-like in that it accepts instructions in the request
+body for what Salt functions to execute, and the response contains the result
+of those function calls.
 
-    This REST interface is flexible in what data formats it will accept as well
-    as what formats it will return (e.g., JSON, YAML, x-www-form-urlencoded).
+For example:
 
-    * Specify the format of data in the request body by including the
-      :mailheader:`Content-Type` header.
-    * Specify the desired data format for the response body with the
-      :mailheader:`Accept` header.
+.. code-block:: text
 
-Data sent in :http:method:`post` and :http:method:`put` requests  must be in
-the format of a list of lowstate dictionaries. This allows multiple commands to
-be executed in a single HTTP request. The order of commands in the request
-corresponds to the return for each command in the response.
+    % curl -sSi https://localhost:8000 \
+        -H 'Content-type: application/json' \
+        -d '[{
+            "client": "local",
+            "tgt": "*",
+            "fun": "test.ping"
+        }]'
+    HTTP/1.1 200 OK
+    Content-Type: application/json
+    [...snip...]
 
-Lowstate, broadly, is a dictionary of values that are mapped to a function
-call. This pattern is used pervasively throughout Salt. The functions called
-from netapi modules are described in :ref:`Client Interfaces <netapi-clients>`.
+    {"return": [{"jerry": true}]}
 
-The following example (in JSON format) causes Salt to execute two commands, a
-command sent to minions as well as a runner function on the master::
+The request body must be an array of commands. Use this workflow to build a
+command:
 
-    [{
-        "client": "local",
-        "tgt": "*",
-        "fun": "test.fib",
-        "arg": ["10"]
-    },
+1.  Choose a client interface.
+2.  Choose a function.
+3.  Fill out the remaining parameters needed for the chosen client.
+
+The ``client`` field is a reference to the main Python classes used in Salt's
+Python API. Read the full :ref:`client interfaces <netapi-clients>`
+documentation, but in short:
+
+* "local" uses :py:class:`LocalClient <salt.client.LocalClient>` which sends
+  commands to Minions. Equivalent to the ``salt`` CLI command.
+* "runner" uses :py:class:`RunnerClient <salt.runner.RunnerClient>` which
+  invokes runner modules on the Master. Equivalent to the ``salt-run`` CLI
+  command.
+* "wheel" uses :py:class:`WheelClient <salt.wheel.WheelClient>` which invokes
+  wheel modules on the Master. Wheel modules do not have a direct CLI
+  equivalent but they typically manage Master-side resources such as state
+  files, pillar files, the Salt config files, and the :py:mod:`key wheel module
+  <salt.wheel.key>` exposes similar functionality as the ``salt-key`` CLI
+  command.
+
+Most clients have variants like synchronous or asynchronous execution as well as
+others like batch execution. See the :ref:`full list of client interfaces
+<netapi-clients>`.
+
+Each client requires different arguments and sometimes has different syntax.
+For example, ``LocalClient`` requires the ``tgt`` argument because it forwards
+the command to Minions and the other client interfaces do not. ``LocalClient``
+also takes ``arg`` (array) and ``kwarg`` (dictionary) arguments because these
+values are sent to the Minions and used to execute the requested function
+there. ``RunnerClient`` and ``WheelClient`` are executed directly on the Master
+and thus do not need or accept those arguments.
+
+Read the method signatures in the client documentation linked above, but
+hopefully an example will help illustrate the concept. This example causes Salt
+to execute two functions -- the :py:func:`test.arg execution function
+<salt.modules.test.arg>` using ``LocalClient`` and the :py:func:`test.arg
+runner function <salt.runners.test.arg>` using ``RunnerClient``; note the
+different structure for each command. The results for both are combined and
+returned as one response.
+
+.. code-block:: text
+
+    % curl -b ~/cookies.txt -sSi localhost:8000 \
+        -H 'Content-type: application/json' \
+        -d '
+    [
+        {
+            "client": "local",
+            "tgt": "*",
+            "fun": "test.arg",
+            "arg": ["positional arg one", "positional arg two"],
+            "kwarg": {
+                "keyword arg one": "Hello from a minion",
+                "keyword arg two": "Hello again from a minion"
+            }
+        },
+        {
+            "client": "runner",
+            "fun": "test.arg",
+            "keyword arg one": "Hello from a master",
+            "keyword arg two": "Runners do not support positional args"
+        }
+    ]
+    '
+    HTTP/1.1 200 OK
+    [...snip...]
     {
-        "client": "runner",
-        "fun": "jobs.lookup_jid",
-        "jid": "20130603122505459265"
-    }]
+      "return": [
+        {
+          "jerry": {
+            "args": [
+              "positional arg one",
+              "positional arg two"
+            ],
+            "kwargs": {
+              "keyword arg one": "Hello from a minion",
+              "keyword arg two": "Hello again from a minion",
+              [...snip...]
+            }
+          },
+          [...snip; other minion returns here...]
+        },
+        {
+          "args": [],
+          "kwargs": {
+            "keyword arg two": "Runners do not support positional args",
+            "keyword arg one": "Hello from a master"
+          }
+        }
+      ]
+    }
 
-.. admonition:: x-www-form-urlencoded
+One more example, this time with more commonly used functions:
 
-    Sending JSON or YAML in the request body is simple and most flexible,
-    however sending data in urlencoded format is also supported with the
-    caveats below. It is the default format for HTML forms, many JavaScript
-    libraries, and the :command:`curl` command.
+.. code-block:: text
 
-    For example, the equivalent to running ``salt '*' test.ping`` is sending
-    ``fun=test.ping&arg&client=local&tgt=*`` in the HTTP request body.
+    curl -b /tmp/cookies.txt -sSi localhost:8000 \
+        -H 'Content-type: application/json' \
+        -d '
+    [
+        {
+            "client": "local",
+            "tgt": "*",
+            "fun": "state.sls",
+            "kwarg": {
+                "mods": "apache",
+                "pillar": {
+                    "lookup": {
+                        "wwwdir": "/srv/httpd/htdocs"
+                    }
+                }
+            }
+        },
+        {
+            "client": "runner",
+            "fun": "cloud.create",
+            "provider": "my-ec2-provider",
+            "instances": "my-centos-6",
+            "image": "ami-1624987f",
+            "delvol_on_destroy", true
+        }
+    ]
+    '
+    HTTP/1.1 200 OK
+    [...snip...]
+    {
+      "return": [
+        {
+          "jerry": {
+            "pkg_|-install_apache_|-httpd_|-installed": {
+                [...snip full state return here...]
+            }
+          }
+          [...snip other minion returns here...]
+        },
+        {
+            [...snip full salt-cloud output here...]
+        }
+      ]
+    }
 
-    Caveats:
+Content negotiation
+-------------------
+
+This REST interface is flexible in what data formats it will accept as well
+as what formats it will return (e.g., JSON, YAML, urlencoded).
+
+* Specify the format of data in the request body by including the
+  :mailheader:`Content-Type` header.
+* Specify the desired data format for the response body with the
+  :mailheader:`Accept` header.
+
+We recommend the JSON format for most HTTP requests. urlencoded data is simple
+and cannot express complex data structures -- and that is often required for
+some Salt commands, such as starting a state run that uses Pillar data. Salt's
+CLI tool can reformat strings passed in at the CLI into complex data
+structures, and that behavior also works via salt-api, but that can be brittle
+and since salt-api can accept JSON it is best just to send JSON.
+
+Here is an example of sending urlencoded data:
+
+.. code-block:: bash
+
+    curl -sSik https://localhost:8000 \\
+        -b ~/cookies.txt \\
+        -d client=runner \\
+        -d fun='jobs.lookup_jid' \\
+        -d jid='20150129182456704682'
+
+.. admonition:: urlencoded data caveats
 
     * Only a single command may be sent per HTTP request.
     * Repeating the ``arg`` parameter multiple times will cause those
@@ -220,9 +384,23 @@ command sent to minions as well as a runner function on the master::
 
       Note, some popular frameworks and languages (notably jQuery, PHP, and
       Ruby on Rails) will automatically append empty brackets onto repeated
-      parameters. E.g., ``arg=one``, ``arg=two`` will be sent as ``arg[]=one``,
-      ``arg[]=two``. This is not supported; send JSON or YAML instead.
+      query string parameters. E.g., ``?foo[]=fooone&foo[]=footwo``. This is
+      **not** supported; send ``?foo=fooone&foo=footwo`` instead, or send JSON
+      or YAML.
 
+    A note about ``curl``
+
+    The ``-d`` flag to curl does *not* automatically urlencode data which can
+    affect passwords and other data that contains characters that must be
+    encoded. Use the ``--data-urlencode`` flag instead. E.g.:
+
+    .. code-block:: bash
+
+        curl -ksi http://localhost:8000/login \\
+        -H "Accept: application/json" \\
+        -d username='myapiuser' \\
+        --data-urlencode password='1234+' \\
+        -d eauth='pam'
 
 .. |req_token| replace:: a session token from :py:class:`~Login`.
 .. |req_accept| replace:: the desired response format.
@@ -234,6 +412,7 @@ command sent to minions as well as a runner function on the master::
 .. |200| replace:: success
 .. |401| replace:: authentication required
 .. |406| replace:: requested Content-Type not available
+
 '''
 # We need a custom pylintrc here...
 # pylint: disable=W0212,E1101,C0103,R0201,W0221,W0613
@@ -245,7 +424,9 @@ import itertools
 import functools
 import logging
 import json
+import os
 import StringIO
+import signal
 import tarfile
 import time
 from multiprocessing import Process, Pipe
@@ -331,7 +512,7 @@ def salt_token_tool():
 
 def salt_api_acl_tool(username, request):
     '''
-    ..versionadded:: Boron
+    ..versionadded:: 2016.3.0
 
     Verifies user requests against the API whitelist. (User/IP pair)
     in order to provide whitelisting for the API similar to the
@@ -509,6 +690,8 @@ def hypermedia_handler(*args, **kwargs):
     except (salt.exceptions.SaltDaemonNotRunning,
             salt.exceptions.SaltReqTimeoutError) as exc:
         raise cherrypy.HTTPError(503, exc.strerror)
+    except (cherrypy.TimeoutError, salt.exceptions.SaltClientTimeout):
+        raise cherrypy.HTTPError(504)
     except cherrypy.CherryPyException:
         raise
     except Exception as exc:
@@ -532,7 +715,12 @@ def hypermedia_handler(*args, **kwargs):
     # Transform the output from the handler into the requested output format
     cherrypy.response.headers['Content-Type'] = best
     out = cherrypy.response.processors[best]
-    return out(ret)
+    try:
+        return out(ret)
+    except Exception:
+        msg = 'Could not serialize the return data from Salt.'
+        logger.debug(msg, exc_info=True)
+        raise cherrypy.HTTPError(500, msg)
 
 
 def hypermedia_out():
@@ -679,7 +867,7 @@ def lowdata_fmt():
     # if the data was sent as urlencoded, we need to make it a list.
     # this is a very forgiving implementation as different clients set different
     # headers for form encoded data (including charset or something similar)
-    if data and not isinstance(data, list):
+    if data and isinstance(data, collections.Mapping):
         # Make the 'arg' param a list if not already
         if 'arg' in data and not isinstance(data['arg'], list):
             data['arg'] = [data['arg']]
@@ -694,12 +882,12 @@ cherrypy.tools.html_override = cherrypy.Tool('on_start_resource',
         html_override_tool, priority=53)
 cherrypy.tools.salt_token = cherrypy.Tool('on_start_resource',
         salt_token_tool, priority=55)
+cherrypy.tools.cors_tool = cherrypy.Tool('before_request_body',
+        cors_tool, priority=50)
 cherrypy.tools.salt_auth = cherrypy.Tool('before_request_body',
         salt_auth_tool, priority=60)
 cherrypy.tools.hypermedia_in = cherrypy.Tool('before_request_body',
         hypermedia_in)
-cherrypy.tools.cors_tool = cherrypy.Tool('before_request_body',
-        cors_tool, priority=30)
 cherrypy.tools.lowdata_fmt = cherrypy.Tool('before_handler',
         lowdata_fmt, priority=40)
 cherrypy.tools.hypermedia_out = cherrypy.Tool('before_handler',
@@ -848,11 +1036,10 @@ class LowDataAdapter(object):
         .. code-block:: bash
 
             curl -sSik https://localhost:8000 \\
-                    -H "Accept: application/x-yaml" \\
-                    -H "X-Auth-Token: d40d1e1e<...snip...>" \\
-                    -d client=local \\
-                    -d tgt='*' \\
-                    -d fun='test.ping' \\
+                -b ~/cookies.txt \\
+                -H "Accept: application/x-yaml" \\
+                -H "Content-type: application/json" \\
+                -d '[{"client": "local", "tgt": "*", "fun": "test.ping"}]'
 
         .. code-block:: http
 
@@ -860,10 +1047,9 @@ class LowDataAdapter(object):
             Host: localhost:8000
             Accept: application/x-yaml
             X-Auth-Token: d40d1e1e
-            Content-Length: 36
-            Content-Type: application/x-www-form-urlencoded
+            Content-Type: application/json
 
-            fun=test.ping&client=local&tgt=*
+            [{"client": "local", "tgt": "*", "fun": "test.ping"}]
 
         **Example response:**
 
@@ -876,55 +1062,10 @@ class LowDataAdapter(object):
 
             return:
             - ms-0: true
-                ms-1: true
-                ms-2: true
-                ms-3: true
-                ms-4: true
-
-        **Other examples**:
-
-        .. code-block:: bash
-
-            # Sending multiple positional args with urlencoded:
-            curl -sSik https://localhost:8000 \\
-                    -d client=local \\
-                    -d tgt='*' \\
-                    -d fun='cmd.run' \\
-                    -d arg='du -sh .' \\
-                    -d arg='/path/to/dir'
-
-            # Sending positional args and Keyword args with JSON:
-            echo '[
-                {
-                    "client": "local",
-                    "tgt": "*",
-                    "fun": "cmd.run",
-                    "arg": [
-                        "du -sh .",
-                        "/path/to/dir"
-                    ],
-                    "kwarg": {
-                        "shell": "/bin/sh",
-                        "template": "jinja"
-                    }
-                }
-            ]' | curl -sSik https://localhost:8000 \\
-                    -H 'Content-type: application/json' \\
-                    -d@-
-
-            # Calling runner functions:
-            curl -sSik https://localhost:8000 \\
-                    -d client=runner \\
-                    -d fun='jobs.lookup_jid' \\
-                    -d jid='20150129182456704682' \\
-                    -d outputter=highstate
-
-            # Calling wheel functions:
-            curl -sSik https://localhost:8000 \\
-                    -d client=wheel \\
-                    -d fun='key.gen_accept' \\
-                    -d id_=dave \\
-                    -d keysize=4096
+              ms-1: true
+              ms-2: true
+              ms-3: true
+              ms-4: true
         '''
         return {
             'return': list(self.exec_lowstate(
@@ -1013,17 +1154,16 @@ class Minions(LowDataAdapter):
         .. code-block:: bash
 
             curl -sSi localhost:8000/minions \\
+                -b ~/cookies.txt \\
                 -H "Accept: application/x-yaml" \\
-                -d tgt='*' \\
-                -d fun='status.diskusage'
+                -d '[{"tgt": "*", "fun": "status.diskusage"}]'
 
         .. code-block:: http
 
             POST /minions HTTP/1.1
             Host: localhost:8000
             Accept: application/x-yaml
-            Content-Length: 26
-            Content-Type: application/x-www-form-urlencoded
+            Content-Type: application/json
 
             tgt=*&fun=status.diskusage
 
@@ -1439,20 +1579,25 @@ class Login(LowDataAdapter):
         .. code-block:: bash
 
             curl -si localhost:8000/login \\
-                    -H "Accept: application/json" \\
-                    -d username='saltuser' \\
-                    -d password='saltpass' \\
-                    -d eauth='pam'
+                -c ~/cookies.txt \\
+                -H "Accept: application/json" \\
+                -H "Content-type: application/json" \\
+                -d '{
+                    "username": "saltuser",
+                    "password": "saltuser",
+                    "eauth": "auto"
+                }'
 
         .. code-block:: http
 
             POST / HTTP/1.1
             Host: localhost:8000
             Content-Length: 42
-            Content-Type: application/x-www-form-urlencoded
+            Content-Type: application/json
             Accept: application/json
 
-            username=saltuser&password=saltpass&eauth=pam
+            {"username": "saltuser", "password": "saltuser", "eauth": "auto"}
+
 
         **Example response:**
 
@@ -1592,12 +1737,15 @@ class Run(LowDataAdapter):
 
             curl -sS localhost:8000/run \\
                 -H 'Accept: application/x-yaml' \\
-                -d client='local' \\
-                -d tgt='*' \\
-                -d fun='test.ping' \\
-                -d username='saltdev' \\
-                -d password='saltdev' \\
-                -d eauth='pam'
+                -H 'Content-type: application/json' \\
+                -d '[{
+                    "client": "local",
+                    "tgt": "*",
+                    "fun": "test.ping",
+                    "username": "saltdev",
+                    "password": "saltdev",
+                    "eauth": "auto"
+                }]'
 
         .. code-block:: http
 
@@ -1605,9 +1753,9 @@ class Run(LowDataAdapter):
             Host: localhost:8000
             Accept: application/x-yaml
             Content-Length: 75
-            Content-Type: application/x-www-form-urlencoded
+            Content-Type: application/json
 
-            client=local&tgt=*&fun=test.ping&username=saltdev&password=saltdev&eauth=pam
+            [{"client": "local", "tgt": "*", "fun": "test.ping", "username": "saltdev", "password": "saltdev", "eauth": "auto"}]
 
         **Example response:**
 
@@ -1619,17 +1767,19 @@ class Run(LowDataAdapter):
 
             return:
             - ms-0: true
-                ms-1: true
-                ms-2: true
-                ms-3: true
-                ms-4: true
+              ms-1: true
+              ms-2: true
+              ms-3: true
+              ms-4: true
 
-        The /run enpoint can also be used to issue commands using the salt-ssh subsystem.
+        The /run enpoint can also be used to issue commands using the salt-ssh
+        subsystem.
 
-        When using salt-ssh, eauth credentials should not be supplied. Instad, authentication
-        should be handled by the SSH layer itself. The use of the salt-ssh client does not
-        require a salt master to be running. Instead, only a roster file must be present
-        in the salt configuration directory.
+        When using salt-ssh, eauth credentials should not be supplied. Instad,
+        authentication should be handled by the SSH layer itself. The use of
+        the salt-ssh client does not require a salt master to be running.
+        Instead, only a roster file must be present in the salt configuration
+        directory.
 
         All SSH client requests are synchronous.
 
@@ -1788,11 +1938,12 @@ class Events(object):
         .. code-block:: javascript
 
             var source = new EventSource('/events');
-            source.onopen = function() { console.debug('opening') };
-            source.onerror = function(e) { console.debug('error!', e) };
-            source.onmessage = function(e) {
-                console.debug('Tag: ', e.data.tag)
-                console.debug('Data: ', e.data.data)
+            source.onopen = function() { console.info('Listening ...') };
+            source.onerror = function(err) { console.error(err) };
+            source.onmessage = function(message) {
+              var saltEvent = JSON.parse(message.data);
+              console.info(saltEvent.tag)
+              console.debug(saltEvent.data)
             };
 
         Or using CORS:
@@ -1857,7 +2008,7 @@ class Events(object):
                     transport=self.opts['transport'],
                     opts=self.opts,
                     listen=True)
-            stream = event.iter_events(full=True)
+            stream = event.iter_events(full=True, auto_reconnect=True)
 
             yield u'retry: {0}\n'.format(400)
 
@@ -2031,8 +2182,14 @@ class WebsocketEndpoint(object):
                     transport=self.opts['transport'],
                     opts=self.opts,
                     listen=True)
-            stream = event.iter_events(full=True)
+            stream = event.iter_events(full=True, auto_reconnect=True)
             SaltInfo = event_processor.SaltInfo(handler)
+
+            def signal_handler(signal, frame):
+                os._exit(0)
+
+            signal.signal(signal.SIGTERM, signal_handler)
+
             while True:
                 data = next(stream)
                 if data:
@@ -2141,16 +2298,18 @@ class Webhook(object):
 
         .. code-block:: bash
 
-            curl -sS localhost:8000/hook -d foo='Foo!' -d bar='Bar!'
+            curl -sS localhost:8000/hook \\
+                -H 'Content-type: application/json' \\
+                -d '{"foo": "Foo!", "bar": "Bar!"}'
 
         .. code-block:: http
 
             POST /hook HTTP/1.1
             Host: localhost:8000
             Content-Length: 16
-            Content-Type: application/x-www-form-urlencoded
+            Content-Type: application/json
 
-            foo=Foo&bar=Bar!
+            {"foo": "Foo!", "bar": "Bar!"}
 
         **Example response**:
 
