@@ -122,8 +122,26 @@ def output(data):
     The HighState Outputter is only meant to be used with the state.highstate
     function, or a function that returns highstate return data.
     '''
-    for host, hostdata in six.iteritems(data):
-        return _format_host(host, hostdata)[0]
+    # If additional information is passed through via the "data" dictionary to
+    # the highstate outputter, such as "outputter" or "retcode", discard it.
+    # We only want the state data that was passed through, if it is wrapped up
+    # in the "data" key, as the orchestrate runner does. See Issue #31330,
+    # pull request #27838, and pull request #27175 for more information.
+    if 'data' in data:
+        data = data.pop('data')
+
+    ret = [
+        _format_host(host, hostdata)[0]
+        for host, hostdata in six.iteritems(data)
+    ]
+    if ret:
+        return "\n".join(ret)
+    log.error(
+        'Data passed to highstate outputter is not a valid highstate return: %s',
+        data
+    )
+    # We should not reach here, but if we do return empty string
+    return ''
 
 
 def _format_host(host, data):
@@ -159,13 +177,17 @@ def _format_host(host, data):
                           .format(hcolor, err, colors)))
     if isinstance(data, dict):
         # Verify that the needed data is present
+        data_tmp = {}
         for tname, info in six.iteritems(data):
-            if isinstance(info, dict) and '__run_num__' not in info:
+            if isinstance(info, dict) and info and '__run_num__' not in info:
                 err = (u'The State execution failed to record the order '
                        'in which all states were executed. The state '
                        'return missing data is:')
                 hstrs.insert(0, pprint.pformat(info))
                 hstrs.insert(0, err)
+            if isinstance(info, dict) and 'result' in info:
+                data_tmp[tname] = info
+        data = data_tmp
         # Everything rendered as it should display the output
         for tname in sorted(
                 data,
@@ -271,7 +293,7 @@ def _format_host(host, data):
                 u'    {tcolor}  Result: {ret[result]!s}{colors[ENDC]}',
                 u'    {tcolor} Comment: {comment}{colors[ENDC]}',
             ]
-            if __opts__.get('state_output_profile', True):
+            if __opts__.get('state_output_profile', True) and 'start_time' in ret:
                 state_lines.extend([
                     u'    {tcolor} Started: {ret[start_time]!s}{colors[ENDC]}',
                     u'    {tcolor}Duration: {ret[duration]!s}{colors[ENDC]}',
@@ -502,15 +524,25 @@ def _format_terse(tcolor, comps, ret, colors, tabular):
     elif ret['result'] is None:
         result = u'Differs'
     if tabular is True:
-        fmt_string = u'{0}'
-        if __opts__.get('state_output_profile', True):
+        fmt_string = ''
+        if 'warnings' in ret:
+            fmt_string += u'{c[LIGHT_RED]}Warnings:\n{w}{c[ENDC]}\n'.format(
+                c=colors, w='\n'.join(ret['warnings'])
+            )
+        fmt_string += u'{0}'
+        if __opts__.get('state_output_profile', True) and 'start_time' in ret:
             fmt_string += u'{6[start_time]!s} [{6[duration]!s} ms] '
         fmt_string += u'{2:>10}.{3:<10} {4:7}   Name: {1}{5}'
     elif isinstance(tabular, str):
         fmt_string = tabular
     else:
-        fmt_string = u' {0} Name: {1} - Function: {2}.{3} - Result: {4}'
-        if __opts__.get('state_output_profile', True):
+        fmt_string = ''
+        if 'warnings' in ret:
+            fmt_string += u'{c[LIGHT_RED]}Warnings:\n{w}{c[ENDC]}'.format(
+                c=colors, w='\n'.join(ret['warnings'])
+            )
+        fmt_string += u' {0} Name: {1} - Function: {2}.{3} - Result: {4}'
+        if __opts__.get('state_output_profile', True) and 'start_time' in ret:
             fmt_string += u' Started: - {6[start_time]!s} Duration: {6[duration]!s} ms'
         fmt_string += u'{5}'
 
