@@ -30,7 +30,7 @@ from salt.exceptions import SaltClientError
 FINGERPRINT_REGEX = re.compile(r'^([a-f0-9]{2}:){15}([a-f0-9]{2})$')
 
 
-def status(output=True):
+def status(output=True, timeout=None, gather_job_timeout=None):
     '''
     Print the status of all known salt minions
 
@@ -42,8 +42,14 @@ def status(output=True):
     '''
     ret = {}
     client = salt.client.get_local_client(__opts__['conf_file'])
+
+    if not timeout:
+        timeout = __opts__['timeout']
+    if not gather_job_timeout:
+        gather_job_timeout = __opts__['gather_job_timeout']
+
     try:
-        minions = client.cmd('*', 'test.ping', timeout=__opts__['timeout'])
+        minions = client.cmd('*', 'test.ping', timeout=timeout, gather_job_timeout=gather_job_timeout)
     except SaltClientError as client_error:
         print(client_error)
         return ret
@@ -128,7 +134,7 @@ def down(removekeys=False):
     return ret
 
 
-def up():  # pylint: disable=C0103
+def up(timeout=None, gather_job_timeout=None):  # pylint: disable=C0103
     '''
     Print a list of all of the minions that are up
 
@@ -137,8 +143,13 @@ def up():  # pylint: disable=C0103
     .. code-block:: bash
 
         salt-run manage.up
+        salt-run manage.up timeout=5 gather_job_timeout=5
     '''
-    ret = status(output=False).get('up', [])
+    ret = status(
+        output=False,
+        timeout=timeout,
+        gather_job_timeout=gather_job_timeout
+    ).get('up', [])
     return ret
 
 
@@ -181,7 +192,7 @@ def list_state(subset=None, show_ipv4=False, state=None):
         # Always return 'present' for 0MQ for now
         # TODO: implement other states spport for 0MQ
         ckminions = salt.utils.minions.CkMinions(__opts__)
-        minions = ckminions.connected_ids(show_ipv4=show_ipv4, subset=subset)
+        minions = ckminions.connected_ids(show_ipv4=show_ipv4, subset=subset, include_localhost=True)
 
     connected = dict(minions) if show_ipv4 else sorted(minions)
 
@@ -652,7 +663,7 @@ def bootstrap(version='develop',
                         ('root@' if root_user else '') + host,
                         'python -c \'import urllib; '
                         'print urllib.urlopen('
-                        '\'' + script + '\''
+                        '"' + script + '"'
                         ').read()\' | sh -s -- git ' + version])
 
 
@@ -675,7 +686,7 @@ def bootstrap_psexec(hosts='', master=None, version=None, arch='win32',
 
     installer_url
         URL of minion installer executable. Defaults to the latest version from
-        http://docs.saltstack.com/downloads
+        https://repo.saltstack.com/windows/
 
     username
         Optional user name for login on remote computer.
@@ -694,7 +705,7 @@ def bootstrap_psexec(hosts='', master=None, version=None, arch='win32',
     '''
 
     if not installer_url:
-        base_url = 'http://docs.saltstack.com/downloads/'
+        base_url = 'https://repo.saltstack.com/windows/'
         source = _urlopen(base_url).read()
         salty_rx = re.compile('>(Salt-Minion-(.+?)-(.+)-Setup.exe)</a></td><td align="right">(.*?)\\s*<')
         source_list = sorted([[path, ver, plat, time.strptime(date, "%d-%b-%Y %H:%M")]
