@@ -51,9 +51,16 @@ passed in as a dict, or as a string to pull from pillars or minion config:
                   elb_protocol: HTTPS
                   instance_protocol: HTTP
                   certificate: 'arn:aws:iam::1111111:server-certificate/mycert'
+                  policies:
+                      - my-ssl-policy
+                      - cookie-policy
                 - elb_port: 8210
                   instance_port: 8210
                   elb_protocol: TCP
+            - backends:
+                - instance_port: 80
+                  policies:
+                      - enable-proxy-protocol
             - health_check:
                 target: 'HTTP:80/'
             - attributes:
@@ -74,6 +81,21 @@ passed in as a dict, or as a string to pull from pillars or minion config:
                   zone: example.com.
             - security_groups:
                 - my-security-group
+            - policies:
+                - policy_name: my-ssl-policy
+                  policy_type: SSLNegotiationPolicyType
+                  policy:
+                    Protocol-TLSv1.2: true
+                    Protocol-SSLv3: false
+                    Server-Defined-Cipher-Order: true
+                    ECDHE-ECDSA-AES128-GCM-SHA256: true
+                - policy_name: cookie-policy
+                  policy_type: LBCookieStickinessPolicyType
+                  policy: {}  # no policy means this is a session cookie
+                - policy_name: enable-proxy-protocol
+                  policy_type: ProxyProtocolPolicyType
+                  policy:
+                    ProxyProtocol: true
 
     # Using a profile from pillars
     Ensure myelb ELB exists:
@@ -196,12 +218,29 @@ Overriding the alarm values on the resource:
                 UnHealthyHostCount:
                   attributes:
                     threshold: 2.0
+
+Tags can also be set:
+
+.. versionadded:: 2016.3.0
+
+.. code-block:: yaml
+
+    Ensure myelb ELB exists:
+        boto_elb.present:
+            - name: myelb
+            - region: us-east-1
+            - profile: myelbprofile
+            - tags:
+                MyTag: 'My Tag Value'
+                OtherTag: 'My Other Value'
 '''
 
 # Import Python Libs
 from __future__ import absolute_import
 
 # Import Salt Libs
+import hashlib
+import re
 import salt.utils.dictupdate as dictupdate
 from salt.exceptions import SaltInvocationError
 import salt.ext.six as six
@@ -227,11 +266,15 @@ def present(
         cnames=None,
         alarms=None,
         alarms_from_pillar="boto_elb_alarms",
+        policies=None,
+        policies_from_pillar="boto_elb_policies",
+        backends=None,
         region=None,
         key=None,
         keyid=None,
         profile=None,
-        wait_for_sync=True):
+        wait_for_sync=True,
+        tags=None):
     '''
     Ensure the ELB exists.
 
@@ -289,9 +332,11 @@ def present(
         state will override those from pillar.
 
     cnames
-        An optional list of cname dicts with attributes: name, zone, ttl, and
-        identifier. If specified, a CNAME record will be created referencing
-        this ELB's public FQDN.
+        A list of cname dicts with attributes needed for the DNS add_record state.
+        By default the boto_route53.add_record state will be used, which requires: name, zone, ttl, and identifier.
+        See the boto_route53 state for information about these attributes.
+        Other DNS modules can be called by specifying the provider keyword.
+        the cnames dict will be passed to the state as kwargs.
 
         See the :mod:`salt.states.boto_route53` state for information about
         these attributes.
@@ -323,6 +368,9 @@ def present(
 
     wait_for_sync
         Wait for an INSYNC change status from Route53.
+
+    tags
+        dict of tags
     '''
 
     # load data from attributes_from_pillar and merge with attributes
@@ -376,28 +424,49 @@ def present(
         )
         if len(lb) > 0:
             for cname in cnames:
-                _ret = __states__['boto_route53.present'](
-                    name=cname.get('name'),
-                    value=lb['dns_name'],
-                    zone=cname.get('zone'),
-                    record_type='CNAME',
-                    identifier=cname.get('identifier', None),
-                    ttl=cname.get('ttl', None),
-                    region=region,
-                    key=key,
-                    keyid=keyid,
-                    profile=profile,
-                    wait_for_sync=wait_for_sync
-                )
+                _ret = None
+                dns_provider = 'boto_route53'
+                cname['record_type'] = 'CNAME'
+                cname['value'] = lb['dns_name']
+                if 'provider' in cname:
+                    dns_provider = cname.pop('provider')
+                if dns_provider == 'boto_route53':
+                    if 'profile' not in cname:
+                        cname['profile'] = profile
+                    if 'key' not in cname:
+                        cname['key'] = key
+                    if 'keyid' not in cname:
+                        cname['keyid'] = keyid
+                    if 'region' not in cname:
+                        cname['region'] = region
+                    if 'wait_for_sync' not in cname:
+                        cname['wait_for_sync'] = wait_for_sync
+                _ret = __states__['.'.join([dns_provider, 'present'])](**cname)
                 ret['changes'] = dictupdate.update(ret['changes'], _ret['changes'])
                 ret['comment'] = ' '.join([ret['comment'], _ret['comment']])
                 if not _ret['result']:
                     ret['result'] = _ret['result']
                     if ret['result'] is False:
                         return ret
+
     _ret = _alarms_present(name, alarms, alarms_from_pillar, region, key, keyid, profile)
     ret['changes'] = dictupdate.update(ret['changes'], _ret['changes'])
     ret['comment'] = ' '.join([ret['comment'], _ret['comment']])
+    if not _ret['result']:
+        ret['result'] = _ret['result']
+        if ret['result'] is False:
+            return ret
+    _ret = _policies_present(name, policies, policies_from_pillar, listeners,
+                             backends, region, key, keyid, profile)
+    ret['changes'] = dictupdate.update(ret['changes'], _ret['changes'])
+    ret['comment'] = ' '.join([ret['comment'], _ret['comment']])
+    if not _ret['result']:
+        ret['result'] = _ret['result']
+        if ret['result'] is False:
+            return ret
+    _ret = _tags_present(name, tags, region, key, keyid, profile)
+    ret['changes'] = dictupdate.update(ret['changes'], _ret['changes'])
+    ret['comment'] = ' '.join([ret['comment'], _ret['comment']])
     if not _ret['result']:
         ret['result'] = _ret['result']
         if ret['result'] is False:
@@ -477,6 +546,9 @@ def register_instances(name, instances, region=None, key=None, keyid=None,
     return ret
 
 
+DEFAULT_PILLAR_LISTENER_POLICY_KEY = 'boto_elb_listener_policies'
+
+
 def _elb_present(
         name,
         availability_zones,
@@ -497,33 +569,36 @@ def _elb_present(
                                   ' exclusive arguments.')
     if not listeners:
         listeners = []
-    _listeners = []
     for listener in listeners:
         if len(listener) < 3:
             raise SaltInvocationError('Listeners must have at minimum port,'
                                       ' instance_port and protocol values in'
                                       ' the provided list.')
-        for config in ('elb_port', 'instance_port', 'elb_protocol'):
-            if not listener.get(config):
-                raise SaltInvocationError(
-                    '{0} is a required value for listeners.'.format(config)
-                )
-
+        if 'elb_port' not in listener:
+            raise SaltInvocationError('elb_port is a required value for'
+                                      ' listeners.')
+        if 'instance_port' not in listener:
+            raise SaltInvocationError('instance_port is a required value for'
+                                      ' listeners.')
+        if 'elb_protocol' not in listener:
+            raise SaltInvocationError('elb_protocol is a required value for'
+                                      ' listeners.')
         listener['elb_protocol'] = listener['elb_protocol'].upper()
         if listener['elb_protocol'] == 'HTTPS' and 'certificate' not in listener:
             raise SaltInvocationError('certificate is a required value for'
                                       ' listeners if HTTPS is set for'
                                       ' elb_protocol.')
-        # We define all listeners as complex listeners.
-        if not listener.get('instance_protocol'):
-            listener['instance_protocol'] = listener['elb_protocol'].upper()
-        else:
-            listener['instance_protocol'] = listener['instance_protocol'].upper()
-        _listener = [listener['elb_port'], listener['instance_port'],
-                     listener['elb_protocol'], listener['instance_protocol']]
-        if 'certificate' in listener:
-            _listener.append(listener['certificate'])
-        _listeners.append(_listener)
+
+        # best attempt at principle of least surprise here:
+        #     only use the default pillar in cases where we don't explicitly
+        #     define policies OR policies_from_pillar on a listener
+        policies = listener.setdefault('policies', [])
+        policies_pillar = listener.get('policies_from_pillar', None)
+        if not policies and policies_pillar is None:
+            policies_pillar = DEFAULT_PILLAR_LISTENER_POLICY_KEY
+        if policies_pillar:
+            policies += __salt__['pillar.get'](policies_pillar, {}).get(listener['elb_protocol'], [])
+
     if subnets:
         vpc_id = __salt__['boto_vpc.get_subnet_association'](
             subnets, region, key, keyid, profile
@@ -533,8 +608,8 @@ def _elb_present(
             msg = 'Subnets {0} do not map to a valid vpc id.'.format(subnets)
             raise SaltInvocationError(msg)
         security_groups = __salt__['boto_secgroup.convert_to_group_ids'](
-            security_groups, vpc_id, region=region, key=key, keyid=keyid,
-            profile=profile
+            security_groups, vpc_id=vpc_id, region=region, key=key,
+            keyid=keyid, profile=profile
         )
         if not security_groups:
             msg = 'Security groups {0} do not map to valid security group ids.'
@@ -547,7 +622,7 @@ def _elb_present(
             ret['result'] = None
             return ret
         created = __salt__['boto_elb.create'](name, availability_zones,
-                                              _listeners, subnets,
+                                              listeners, subnets,
                                               security_groups, scheme, region,
                                               key, keyid, profile)
         if created:
@@ -568,7 +643,7 @@ def _elb_present(
             ret['result'] = _ret['result']
             if ret['result'] is False:
                 return ret
-        _ret = _listeners_present(name, _listeners, region, key, keyid,
+        _ret = _listeners_present(name, listeners, region, key, keyid,
                                   profile)
         ret['changes'] = dictupdate.update(ret['changes'], _ret['changes'])
         ret['comment'] = ' '.join([ret['comment'], _ret['comment']])
@@ -611,45 +686,75 @@ def _listeners_present(
         return ret
     if not listeners:
         listeners = []
+
+    expected_listeners_by_tuple = {}
+    for l in listeners:
+        l_key = __salt__['boto_elb.listener_dict_to_tuple'](l)
+        expected_listeners_by_tuple[l_key] = l
+    actual_listeners_by_tuple = {}
+    for l in lb['listeners']:
+        l_key = __salt__['boto_elb.listener_dict_to_tuple'](l)
+        actual_listeners_by_tuple[l_key] = l
+
     to_delete = []
     to_create = []
-    for listener in listeners:
-        if listener not in lb['listeners']:
-            to_create.append(listener)
-    for listener in lb['listeners']:
-        if listener not in listeners:
-            to_delete.append(listener[0])
-    if to_create or to_delete:
-        if __opts__['test']:
-            msg = 'ELB {0} set to have listeners modified.'.format(name)
+
+    for t, l in expected_listeners_by_tuple.iteritems():
+        if t not in actual_listeners_by_tuple:
+            to_create.append(l)
+    for t, l in actual_listeners_by_tuple.iteritems():
+        if t not in expected_listeners_by_tuple:
+            to_delete.append(l)
+
+    if __opts__['test']:
+        msg = []
+        if to_create or to_delete:
+            msg.append('ELB {0} set to have listeners modified:'.format(name))
+            for listener in to_create:
+                msg.append('Listener {0} added.'.format(
+                        __salt__['boto_elb.listener_dict_to_tuple'](listener)))
+            for listener in to_delete:
+                msg.append('Listener {0} deleted.'.format(
+                        __salt__['boto_elb.listener_dict_to_tuple'](listener)))
+        else:
+            msg.append('Listeners already set on ELB {0}.'.format(name))
+        ret['comment'] = ' '.join(msg)
+        ret['result'] = None
+        return ret
+
+    if to_delete:
+        ports = [l['elb_port'] for l in to_delete]
+        deleted = __salt__['boto_elb.delete_listeners'](name, ports,
+                                                        region, key, keyid,
+                                                        profile)
+        if deleted:
+            ret['comment'] = 'Deleted listeners on {0} ELB.'.format(name)
+        else:
+            msg = 'Failed to delete listeners on {0} ELB.'.format(name)
             ret['comment'] = msg
-            ret['result'] = None
-            return ret
-        if to_delete:
-            deleted = __salt__['boto_elb.delete_listeners'](name, to_delete,
-                                                            region, key, keyid,
-                                                            profile)
-            if deleted:
-                ret['comment'] = 'Deleted listeners on {0} ELB.'.format(name)
-            else:
-                msg = 'Failed to delete listeners on {0} ELB.'.format(name)
-                ret['comment'] = msg
-                ret['result'] = False
-        if to_create:
-            created = __salt__['boto_elb.create_listeners'](name, to_create,
-                                                            region, key, keyid,
-                                                            profile)
-            if created:
-                msg = 'Created listeners on {0} ELB.'
-                ret['comment'] = ' '.join([ret['comment'], msg.format(name)])
-            else:
-                msg = 'Failed to create listeners on {0} ELB.'
-                ret['comment'] = ' '.join([ret['comment'], msg.format(name)])
-                ret['result'] = False
-        ret['changes']['old'] = {'listeners': lb['listeners']}
+            ret['result'] = False
+
+    if to_create:
+        created = __salt__['boto_elb.create_listeners'](name, to_create,
+                                                        region, key, keyid,
+                                                        profile)
+        if created:
+            msg = 'Created listeners on {0} ELB.'
+            ret['comment'] = ' '.join([ret['comment'], msg.format(name)])
+        else:
+            msg = 'Failed to create listeners on {0} ELB.'
+            ret['comment'] = ' '.join([ret['comment'], msg.format(name)])
+            ret['result'] = False
+
+    if to_create or to_delete:
+        ret['changes']['listeners'] = {}
+        ret['changes']['listeners']['old'] = lb['listeners']
         lb = __salt__['boto_elb.get_elb_config'](name, region, key, keyid,
                                                  profile)
-        ret['changes']['new'] = {'listeners': lb['listeners']}
+        ret['changes']['listeners']['new'] = lb['listeners']
+    else:
+        ret['comment'] = 'Listeners already set on ELB {0}.'.format(name)
+
     return ret
 
 
@@ -973,12 +1078,274 @@ def _alarms_present(name, alarms, alarms_from_pillar, region, key, keyid, profil
     return merged_return_value
 
 
+def _policies_present(
+        name,
+        policies,
+        policies_from_pillar,
+        listeners,
+        backends,
+        region,
+        key,
+        keyid,
+        profile):
+    '''helper method for present. ensure that ELB policies are set'''
+    if policies is None:
+        policies = []
+    pillar_policies = __salt__['config.option'](policies_from_pillar, [])
+    policies = policies + pillar_policies
+    if backends is None:
+        backends = []
+
+    # check for policy name uniqueness and correct type
+    policy_names = set()
+    for p in policies:
+        if 'policy_name' not in p:
+            raise SaltInvocationError('policy_name is a required value for '
+                                      'policies.')
+        if 'policy_type' not in p:
+            raise SaltInvocationError('policy_type is a required value for '
+                                      'policies.')
+        if 'policy' not in p:
+            raise SaltInvocationError('policy is a required value for '
+                                      'listeners.')
+        # check for unique policy names
+        if p['policy_name'] in policy_names:
+            raise SaltInvocationError('Policy names must be unique: policy {0}'
+                    ' is declared twice.'.format(p['policy_name']))
+        policy_names.add(p['policy_name'])
+
+    # check that listeners refer to valid policy names
+    for l in listeners:
+        for p in l.get('policies', []):
+            if p not in policy_names:
+                raise SaltInvocationError('Listener {0} on ELB {1} refers to '
+                        'undefined policy {2}.'.format(l['elb_port'], name, p))
+
+    # check that backends refer to valid policy names
+    for b in backends:
+        for p in b.get('policies', []):
+            if p not in policy_names:
+                raise SaltInvocationError('Backend {0} on ELB {1} refers to '
+                        'undefined policy '
+                        '{2}.'.format(b['instance_port'], name, p))
+
+    ret = {'result': True, 'comment': '', 'changes': {}}
+
+    lb = __salt__['boto_elb.get_elb_config'](name, region, key, keyid, profile)
+    if not lb:
+        msg = '{0} ELB configuration could not be retrieved.'.format(name)
+        ret['comment'] = msg
+        ret['result'] = False
+        return ret
+
+    # Policies have two names:
+    # - a short name ('name') that's only the policy name (e.g. testpolicy)
+    # - a canonical name ('cname') that contains the policy type and hash
+    #   (e.g. SSLNegotiationPolicy-testpolicy-14b32f668639cc8ea1391e062af98524)
+
+    policies_by_cname = {}
+    cnames_by_name = {}
+    for p in policies:
+        cname = _policy_cname(p)
+        policies_by_cname[cname] = p
+        cnames_by_name[p['policy_name']] = cname
+
+    expected_policy_names = policies_by_cname.keys()
+    actual_policy_names = lb['policies']
+
+    # This is sadly a huge hack to get around the fact that AWS assigns a
+    # default SSLNegotiationPolicyType policy (with the naming scheme
+    # ELBSecurityPolicy-YYYY-MM) to all ELBs terminating SSL without an
+    # explicit policy set. If we don't keep track of the default policies and
+    # explicitly exclude them from deletion, orchestration will fail because we
+    # attempt to delete the default policy that's being used by listeners that
+    # were created with no explicit policy.
+    default_aws_policies = set()
+
+    expected_policies_by_listener = {}
+    for l in listeners:
+        expected_policies_by_listener[l['elb_port']] = set(
+                [cnames_by_name[p] for p in l.get('policies', [])])
+
+    actual_policies_by_listener = {}
+    for l in lb['listeners']:
+        listener_policies = set(l.get('policies', []))
+        actual_policies_by_listener[l['elb_port']] = listener_policies
+        # Determine if any actual listener policies look like default policies,
+        # so we can exclude them from deletion below (see note about this hack
+        # above).
+        for p in listener_policies:
+            if re.match(r'^ELBSecurityPolicy-\d{4}-\d{2}$', p):
+                default_aws_policies.add(p)
+
+    expected_policies_by_backend = {}
+    for b in backends:
+        expected_policies_by_backend[b['instance_port']] = set(
+                [cnames_by_name[p] for p in b.get('policies', [])])
+
+    actual_policies_by_backend = {}
+    for b in lb['backends']:
+        backend_policies = set(b.get('policies', []))
+        actual_policies_by_backend[b['instance_port']] = backend_policies
+
+    to_delete = []
+    to_create = []
+
+    for policy_name in expected_policy_names:
+        if policy_name not in actual_policy_names:
+            to_create.append(policy_name)
+    for policy_name in actual_policy_names:
+        if policy_name not in expected_policy_names:
+            if policy_name not in default_aws_policies:
+                to_delete.append(policy_name)
+
+    listeners_to_update = set()
+    for port, policies in expected_policies_by_listener.iteritems():
+        if policies != actual_policies_by_listener.get(port, set()):
+            listeners_to_update.add(port)
+    for port, policies in actual_policies_by_listener.iteritems():
+        if policies != expected_policies_by_listener.get(port, set()):
+            listeners_to_update.add(port)
+
+    backends_to_update = set()
+    for port, policies in expected_policies_by_backend.iteritems():
+        if policies != actual_policies_by_backend.get(port, set()):
+            backends_to_update.add(port)
+    for port, policies in actual_policies_by_backend.iteritems():
+        if policies != expected_policies_by_backend.get(port, set()):
+            backends_to_update.add(port)
+
+    if __opts__['test']:
+        msg = []
+        if to_create or to_delete:
+            msg.append('ELB {0} set to have policies modified:'.format(name))
+            for policy in to_create:
+                msg.append('Policy {0} added.'.format(policy))
+            for policy in to_delete:
+                msg.append('Policy {0} deleted.'.format(policy))
+            ret['result'] = None
+        else:
+            msg.append('Policies already set on ELB {0}.'.format(name))
+        for listener in listeners_to_update:
+            msg.append('Listener {0} policies updated.'.format(listener))
+            ret['result'] = None
+        for backend in backends_to_update:
+            msg.append('Backend {0} policies updated.'.format(backend))
+            ret['result'] = None
+        ret['comment'] = ' '.join(msg)
+        return ret
+
+    if to_create:
+        for policy_name in to_create:
+            created = __salt__['boto_elb.create_policy'](
+                name=name,
+                policy_name=policy_name,
+                policy_type=policies_by_cname[policy_name]['policy_type'],
+                policy=policies_by_cname[policy_name]['policy'],
+                region=region,
+                key=key,
+                keyid=keyid,
+                profile=profile)
+            if created:
+                ret['changes'].setdefault(policy_name, {})['new'] = policy_name
+                comment = "Policy {0} was created on ELB {1}".format(
+                        policy_name, name)
+                ret['comment'] = ' '.join([ret['comment'], comment])
+                ret['result'] = True
+            else:
+                ret['result'] = False
+                return ret
+
+    for port in listeners_to_update:
+        policy_set = __salt__['boto_elb.set_listener_policy'](
+                name=name,
+                port=port,
+                policies=list(expected_policies_by_listener.get(port, [])),
+                region=region,
+                key=key,
+                keyid=keyid,
+                profile=profile)
+        if policy_set:
+            policy_key = 'listener_{0}_policy'.format(port)
+            ret['changes'][policy_key] = {
+                    'old': list(actual_policies_by_listener.get(port, [])),
+                    'new': list(expected_policies_by_listener.get(port, [])),
+                    }
+            comment = "Policy {0} was created on ELB {1} listener {2}".format(
+                    expected_policies_by_listener[port], name, port)
+            ret['comment'] = ' '.join([ret['comment'], comment])
+            ret['result'] = True
+        else:
+            ret['result'] = False
+            return ret
+
+    for port in backends_to_update:
+        policy_set = __salt__['boto_elb.set_backend_policy'](
+                name=name,
+                port=port,
+                policies=list(expected_policies_by_backend.get(port, [])),
+                region=region,
+                key=key,
+                keyid=keyid,
+                profile=profile)
+        if policy_set:
+            policy_key = 'backend_{0}_policy'.format(port)
+            ret['changes'][policy_key] = {
+                    'old': list(actual_policies_by_backend.get(port, [])),
+                    'new': list(expected_policies_by_backend.get(port, [])),
+                    }
+            comment = "Policy {0} was created on ELB {1} backend {2}".format(
+                    expected_policies_by_backend[port], name, port)
+            ret['comment'] = ' '.join([ret['comment'], comment])
+            ret['result'] = True
+        else:
+            ret['result'] = False
+            return ret
+
+    if to_delete:
+        for policy_name in to_delete:
+            deleted = __salt__['boto_elb.delete_policy'](
+                name=name,
+                policy_name=policy_name,
+                region=region,
+                key=key,
+                keyid=keyid,
+                profile=profile)
+            if deleted:
+                ret['changes'].setdefault(policy_name, {})['old'] = policy_name
+                comment = "Policy {0} was deleted from ELB {1}".format(
+                        policy_name, name)
+                ret['comment'] = ' '.join([ret['comment'], comment])
+                ret['result'] = True
+            else:
+                ret['result'] = False
+                return ret
+    return ret
+
+
+def _policy_cname(policy_dict):
+    policy_name = policy_dict['policy_name']
+    policy_type = policy_dict['policy_type']
+    policy = policy_dict['policy']
+    canonical_policy_repr = str(sorted(list(policy.iteritems()), key=lambda x: str(x[0])))
+    policy_hash = hashlib.md5(str(canonical_policy_repr)).hexdigest()
+    if policy_type.endswith('Type'):
+        policy_type = policy_type[:-4]
+    return "{0}-{1}-{2}".format(policy_type, policy_name, policy_hash)
+
+
 def absent(
         name,
         region=None,
         key=None,
         keyid=None,
         profile=None):
+    '''
+    Ensure an ELB does not exist
+
+    name
+        name of the ELB
+    '''
     ret = {'name': name, 'result': True, 'comment': '', 'changes': {}}
 
     exists = __salt__['boto_elb.exists'](name, region, key, keyid, profile)
@@ -999,3 +1366,95 @@ def absent(
     else:
         ret['comment'] = '{0} ELB does not exist.'.format(name)
     return ret
+
+
+def _tags_present(name,
+                  tags,
+                  region,
+                  key,
+                  keyid,
+                  profile):
+    '''
+    helper function to validate tags on elb
+    '''
+    ret = {'result': True, 'comment': '', 'changes': {}}
+    if tags:
+        lb = __salt__['boto_elb.get_elb_config'](name, region, key, keyid, profile)
+
+        tags_to_add = tags
+        tags_to_update = {}
+        tags_to_remove = []
+        if lb['tags']:
+            for _tag in lb['tags'].keys():
+                if _tag not in tags.keys():
+                    if _tag not in tags_to_remove:
+                        tags_to_remove.append(_tag)
+                else:
+                    if tags[_tag] != lb['tags'][_tag]:
+                        tags_to_update[_tag] = tags[_tag]
+                    tags_to_add.pop(_tag)
+        if tags_to_remove:
+            if __opts__['test']:
+                msg = 'The following tag{0} set to be removed: {1}.'.format(
+                        ('s are' if len(tags_to_remove) > 1 else ' is'), ', '.join(tags_to_remove))
+                ret['comment'] = ' '.join([ret['comment'], msg])
+                ret['result'] = None
+            else:
+                _ret = __salt__['boto_elb.delete_tags'](
+                            name,
+                            tags_to_remove,
+                            region,
+                            key,
+                            keyid,
+                            profile)
+                if not _ret:
+                    ret['result'] = False
+                    msg = 'Error attempting to delete tag {0}.'.format(tags_to_remove)
+                    ret['comment'] = ' '.join([ret['comment'], msg])
+                    return ret
+                if 'old' not in ret['changes']:
+                    ret['changes'] = dictupdate.update(ret['changes'], {'old': {'tags': {}}})
+                for _tag in tags_to_remove:
+                    ret['changes']['old']['tags'][_tag] = lb['tags'][_tag]
+        if tags_to_add or tags_to_update:
+            if __opts__['test']:
+                if tags_to_add:
+                    msg = 'The following tag{0} set to be added: {1}.'.format(
+                            ('s are' if len(tags_to_add.keys()) > 1 else ' is'),
+                            ', '.join(tags_to_add.keys()))
+                    ret['comment'] = ' '. join([ret['comment'], msg])
+                    ret['result'] = None
+                if tags_to_update:
+                    msg = 'The following tag {0} set to be updated: {1}.'.format(
+                            ('values are' if len(tags_to_update.keys()) > 1 else 'value is'),
+                            ', '.join(tags_to_update.keys()))
+                    ret['comment'] = ' '.join([ret['comment'], msg])
+                    ret['result'] = None
+            else:
+                all_tag_changes = dictupdate.update(tags_to_add, tags_to_update)
+                _ret = __salt__['boto_elb.set_tags'](
+                            name,
+                            all_tag_changes,
+                            region,
+                            key,
+                            keyid,
+                            profile)
+                if not _ret:
+                    ret['result'] = False
+                    msg = 'Error attempting to set tags.'
+                    ret['comment'] = ' '.join([ret['comment'], msg])
+                    return ret
+                if 'old' not in ret['changes']:
+                    ret['changes'] = dictupdate.update(ret['changes'], {'old': {'tags': {}}})
+                if 'new' not in ret['changes']:
+                    ret['changes'] = dictupdate.update(ret['changes'], {'new': {'tags': {}}})
+                for tag in all_tag_changes:
+                    ret['changes']['new']['tags'][tag] = tags[tag]
+                    if 'tags' in lb:
+                        if lb['tags']:
+                            if tag in lb['tags']:
+                                ret['changes']['old']['tags'][tag] = lb['tags'][tag]
+        if not tags_to_update and not tags_to_remove and not tags_to_add:
+            msg = 'Tags are already set.'
+            ret['comment'] = ' '.join([ret['comment'], msg])
+    return ret
