@@ -15,7 +15,7 @@ state:
 
 Note that this example is probably unnecessary to use in practice, since the
 ``mine_functions`` and ``mine_interval`` config parameters can be used to
-schedule updates for the mine (see :doc:`here </topics/mine/index>` for more
+schedule updates for the mine (see :ref:`here <salt-mine>` for more
 info).
 
 It is sometimes desirable to trigger a function call after a state is executed,
@@ -29,10 +29,32 @@ for this the :mod:`module.wait <salt.states.module.wait>` state can be used:
         - watch:
           - file: /etc/network/interfaces
 
-All arguments are passed through to the module function being executed.
-However, due to how the state system works, if a module function accepts an
+All arguments that the ``module`` state does not consume are passed through to
+the execution module function being executed:
+
+.. code-block:: yaml
+
+    fetch_out_of_band:
+      module.run:
+        - name: git.fetch
+        - cwd: /path/to/my/repo
+        - user: myuser
+        - opts: '--all'
+
+Due to how the state system works, if a module function accepts an
 argument called, ``name``, then ``m_name`` must be used to specify that
-argument, to avoid a collision with the ``name`` argument. For example:
+argument, to avoid a collision with the ``name`` argument.
+
+Here is a list of keywords hidden by the state system, which must be prefixed
+with ``m_``:
+
+* fun
+* name
+* names
+* state
+* saltenv
+
+For example:
 
 .. code-block:: yaml
 
@@ -40,13 +62,43 @@ argument, to avoid a collision with the ``name`` argument. For example:
       module.run:
         - name: service.disable
         - m_name: nfs
+
+Note that some modules read all or some of the arguments from a list of keyword
+arguments. For example:
+
+.. code-block:: yaml
+
+    mine.send:
+      module.run:
+        - func: network.ip_addrs
+        - kwargs:
+            interface: eth0
+
+.. code-block:: yaml
+
+    cloud.create:
+      module.run:
+        - func: cloud.create
+        - provider: test-provider
+        - m_names:
+          - test-vlad
+        - kwargs: {
+              ssh_username: 'ubuntu',
+              image: 'ami-8d6d9daa',
+              securitygroup: 'default',
+              size: 'c3.large',
+              location: 'ap-northeast-1',
+              delvol_on_destroy: 'True'
+          }
+
 '''
-# Import python libs
-import datetime
+from __future__ import absolute_import
 
 # Import salt libs
 import salt.loader
 import salt.utils
+import salt.utils.jid
+from salt.ext.six.moves import range
 
 
 def wait(name, **kwargs):
@@ -64,11 +116,11 @@ def wait(name, **kwargs):
         return ``True`` but not actually execute, unless one of the following
         two things happens:
 
-        1. The state has a :doc:`watch requisite </ref/states/requisites>`, and
+        1. The state has a :ref:`watch requisite <requisites-watch>`, and
            the state which it is watching changes.
 
-        2. Another state has a :doc:`watch_in requisite
-           </ref/states/requisites>` which references this state, and the state
+        2. Another state has a :ref:`watch_in requisite
+           <requisites-watch-in>` which references this state, and the state
            wth the ``watch_in`` changes.
     '''
     return {'name': name,
@@ -77,7 +129,7 @@ def wait(name, **kwargs):
             'comment': ''}
 
 # Alias module.watch to module.wait
-watch = wait
+watch = salt.utils.alias_function(wait, 'watch')
 
 
 def run(name, **kwargs):
@@ -106,8 +158,7 @@ def run(name, **kwargs):
         ret['comment'] = 'Module function {0} is set to execute'.format(name)
         return ret
 
-    aspec = salt.utils.get_function_argspec(__salt__[name])
-
+    aspec = salt.utils.args.get_function_argspec(__salt__[name])
     args = []
     defaults = {}
 
@@ -130,6 +181,12 @@ def run(name, **kwargs):
         elif arg == 'fun':
             if 'm_fun' in kwargs:
                 defaults[arg] = kwargs.pop('m_fun')
+        elif arg == 'state':
+            if 'm_state' in kwargs:
+                defaults[arg] = kwargs.pop('m_state')
+        elif arg == 'saltenv':
+            if 'm_saltenv' in kwargs:
+                defaults[arg] = kwargs.pop('m_saltenv')
         if arg in kwargs:
             defaults[arg] = kwargs.pop(arg)
     missing = set()
@@ -138,6 +195,12 @@ def run(name, **kwargs):
             rarg = 'm_name'
         elif arg == 'fun':
             rarg = 'm_fun'
+        elif arg == 'names':
+            rarg = 'm_names'
+        elif arg == 'state':
+            rarg = 'm_state'
+        elif arg == 'saltenv':
+            rarg = 'm_saltenv'
         else:
             rarg = arg
         if rarg not in kwargs and arg not in defaults:
@@ -181,12 +244,12 @@ def run(name, **kwargs):
             mret = __salt__[name](*args, **nkwargs)
         else:
             mret = __salt__[name](*args)
-    except Exception:
-        ret['comment'] = 'Module function {0} threw an exception'.format(name)
+    except Exception as e:
+        ret['comment'] = 'Module function {0} threw an exception. Exception: {1}'.format(name, e)
         ret['result'] = False
         return ret
     else:
-        if mret:
+        if mret is not None or mret is not {}:
             ret['changes']['ret'] = mret
 
     if 'returner' in kwargs:
@@ -194,14 +257,26 @@ def run(name, **kwargs):
                 'id': __opts__['id'],
                 'ret': mret,
                 'fun': name,
-                'jid': '{0:%Y%m%d%H%M%S%f}'.format(datetime.datetime.now())}
+                'jid': salt.utils.jid.gen_jid()}
         returners = salt.loader.returners(__opts__, __salt__)
         if kwargs['returner'] in returners:
             returners[kwargs['returner']](ret_ret)
     ret['comment'] = 'Module function {0} executed'.format(name)
+
     ret['result'] = True
-    if ret['changes'].get('retcode', 0) != 0:
+    # if mret is a dict and there is retcode and its non-zero
+    if isinstance(mret, dict) and mret.get('retcode', 0) != 0:
         ret['result'] = False
+    # if its a boolean, return that as the result
+    elif isinstance(mret, bool):
+        ret['result'] = mret
+    else:
+        changes_ret = ret['changes'].get('ret', {})
+        if isinstance(changes_ret, dict):
+            if isinstance(changes_ret.get('result', {}), bool):
+                ret['result'] = changes_ret.get('result', {})
+            elif changes_ret.get('retcode', 0) != 0:
+                ret['result'] = False
     return ret
 
-mod_watch = run  # pylint: disable=C0103
+mod_watch = salt.utils.alias_function(run, 'mod_watch')
