@@ -85,9 +85,9 @@ these states. Here is some example SLS:
     installed if it is not present.
 
 '''
-from __future__ import absolute_import
 
-# Import python libs
+# Import Python libs
+from __future__ import absolute_import
 import sys
 
 # Import salt libs
@@ -112,7 +112,7 @@ def managed(name, ppa=None, **kwargs):
     <salt.modules.yumpkg>`, :mod:`apt <salt.modules.aptpkg>`, and :mod:`zypper
     <salt.modules.zypper>` repositories are supported.
 
-    **YUM OR ZYPPER-BASED SYSTEMS**
+    **YUM/DNF/ZYPPER-BASED SYSTEMS**
 
     .. note::
         One of ``baseurl`` or ``mirrorlist`` below is required. Additionally,
@@ -126,6 +126,16 @@ def managed(name, ppa=None, **kwargs):
         repo. Secondly, it will be the name of the file as stored in
         /etc/yum.repos.d (e.g. ``/etc/yum.repos.d/foo.conf``).
 
+    enabled : True
+        Whether or not the repo is enabled. Can be specified as True/False or
+        1/0.
+
+    disabled : False
+        Included to reduce confusion due to APT's use of the ``disabled``
+        argument. If this is passed for a yum/dnf/zypper-based distro, then the
+        reverse will be passed as ``enabled``. For example passing
+        ``disabled=True`` will assume ``enabled=False``.
+
     humanname
         This is used as the "name" value in the repo file in
         ``/etc/yum.repos.d/`` (or ``/etc/zypp/repos.d`` for Suse distros).
@@ -203,10 +213,16 @@ def managed(name, ppa=None, **kwargs):
                 'deb http://us.archive.ubuntu.com/ubuntu precise main':
                   pkgrepo.managed
 
-    disabled
+    disabled : False
         Toggles whether or not the repo is used for resolving dependencies
         and/or installing packages.
 
+    enabled : True
+        Included to reduce confusion due to yum/dnf/zypper's use of the
+        ``enabled`` argument. If this is passed for an APT-based distro, then
+        the reverse will be passed as ``disabled``. For example, passing
+        ``enabled=False`` will assume ``disabled=False``.
+
     comps
         On apt-based systems, comps dictate the types of packages to be
         installed from the repository (e.g. main, nonfree, ...).  For
@@ -270,6 +286,10 @@ def managed(name, ppa=None, **kwargs):
         ret['comment'] = 'Repo management not implemented on this platform'
         return ret
 
+    if 'key_url' in kwargs and ('keyid' in kwargs or 'keyserver' in kwargs):
+        ret['result'] = False
+        ret['comment'] = 'You may not use both "keyid"/"keyserver" and ' \
+                         '"key_url" argument.'
     if 'repo' in kwargs:
         ret['result'] = False
         ret['comment'] = ('\'repo\' is not a supported argument for this '
@@ -277,8 +297,20 @@ def managed(name, ppa=None, **kwargs):
                           'intended.')
         return ret
 
+    enabled = kwargs.pop('enabled', None)
+    disabled = kwargs.pop('disabled', None)
+
+    if enabled is not None and disabled is not None:
+        ret['result'] = False
+        ret['comment'] = 'Only one of enabled/disabled is allowed'
+        return ret
+    elif enabled is None and disabled is None:
+        # If neither argument was passed we assume the repo will be enabled
+        enabled = True
+
     repo = name
-    if __grains__['os'] == 'Ubuntu':
+    os_family = __grains__['os_family'].lower()
+    if __grains__['os'] in ('Ubuntu', 'Mint'):
         if ppa is not None:
             # overload the name/repo value for PPAs cleanly
             # this allows us to have one code-path for PPAs
@@ -287,30 +319,26 @@ def managed(name, ppa=None, **kwargs):
             except TypeError:
                 repo = ':'.join(('ppa', str(ppa)))
 
-    elif __grains__['os_family'].lower() in ('redhat', 'suse'):
+        kwargs['disabled'] = not salt.utils.is_true(enabled) \
+            if enabled is not None \
+            else salt.utils.is_true(disabled)
+
+    elif os_family in ('redhat', 'suse'):
         if 'humanname' in kwargs:
             kwargs['name'] = kwargs.pop('humanname')
-        _val = lambda x: '1' if salt.utils.is_true(x) else '0'
-        if 'disabled' in kwargs:
-            if 'enabled' in kwargs:
-                ret['result'] = False
-                ret['comment'] = 'Only one of enabled/disabled is permitted'
-                return ret
-            _reverse = lambda x: '1' if x == '0' else '0'
-            kwargs['enabled'] = _reverse(_val(kwargs.pop('disabled')))
-        elif 'enabled' in kwargs:
-            kwargs['enabled'] = _val(kwargs['enabled'])
         if 'name' not in kwargs:
             # Fall back to the repo name if humanname not provided
             kwargs['name'] = repo
 
-    if kwargs.pop('enabled', None):
-        kwargs['disabled'] = False
-        salt.utils.warn_until(
-            'Boron',
-            'The `enabled` argument has been deprecated in favor of '
-            '`disabled`.'
-        )
+        kwargs['enabled'] = not salt.utils.is_true(disabled) \
+            if disabled is not None \
+            else salt.utils.is_true(enabled)
+
+    elif os_family == 'nilinuxrt':
+        # opkg is the pkg virtual
+        kwargs['enabled'] = not salt.utils.is_true(disabled) \
+            if disabled is not None \
+            else salt.utils.is_true(enabled)
 
     for kwarg in _STATE_INTERNAL_KEYWORDS:
         kwargs.pop(kwarg, None)
@@ -335,11 +363,10 @@ def managed(name, ppa=None, **kwargs):
     else:
         sanitizedkwargs = kwargs
 
-    if __grains__['os_family'] == 'Debian':
+    if os_family == 'debian':
         repo = _strip_uri(repo)
 
     if pre:
-        needs_update = False
         for kwarg in sanitizedkwargs:
             if kwarg not in pre:
                 if kwarg == 'enabled':
@@ -347,33 +374,40 @@ def managed(name, ppa=None, **kwargs):
                     # not explicitly set, so we don't need to update the repo
                     # if it's desired to be enabled and the 'enabled' key is
                     # missing from the repo definition
-                    if __grains__['os_family'] == 'RedHat':
+                    if os_family == 'redhat':
                         if not salt.utils.is_true(sanitizedkwargs[kwarg]):
-                            needs_update = True
+                            break
                     else:
-                        needs_update = True
+                        break
                 else:
-                    needs_update = True
+                    break
             elif kwarg == 'comps':
                 if sorted(sanitizedkwargs[kwarg]) != sorted(pre[kwarg]):
-                    needs_update = True
-            elif kwarg == 'line' and __grains__['os_family'] == 'Debian':
+                    break
+            elif kwarg == 'line' and os_family == 'debian':
                 # split the line and sort everything after the URL
                 sanitizedsplit = sanitizedkwargs[kwarg].split()
                 sanitizedsplit[3:] = sorted(sanitizedsplit[3:])
                 reposplit = pre[kwarg].split()
                 reposplit[3:] = sorted(reposplit[3:])
                 if sanitizedsplit != reposplit:
-                    needs_update = True
+                    break
                 if 'comments' in kwargs:
                     _line = pre[kwarg].split('#')
                     if str(kwargs['comments']) not in _line:
-                        needs_update = True
+                        break
             else:
-                if str(sanitizedkwargs[kwarg]) != str(pre[kwarg]):
-                    needs_update = True
-
-        if not needs_update:
+                if os_family in ('redhat', 'suse') \
+                        and any(isinstance(x, bool) for x in
+                                (sanitizedkwargs[kwarg], pre[kwarg])):
+                    # This check disambiguates 1/0 from True/False
+                    if salt.utils.is_true(sanitizedkwargs[kwarg]) != \
+                            salt.utils.is_true(pre[kwarg]):
+                        break
+                else:
+                    if str(sanitizedkwargs[kwarg]) != str(pre[kwarg]):
+                        break
+        else:
             ret['result'] = True
             ret['comment'] = ('Package repo \'{0}\' already configured'
                               .format(name))
@@ -394,7 +428,7 @@ def managed(name, ppa=None, **kwargs):
             pass
 
     try:
-        if __grains__['os_family'] == 'Debian':
+        if os_family == 'debian':
             __salt__['pkg.mod_repo'](repo, saltenv=__env__, **kwargs)
         else:
             __salt__['pkg.mod_repo'](repo, **kwargs)
@@ -425,7 +459,7 @@ def managed(name, ppa=None, **kwargs):
     except Exception as exc:
         ret['result'] = False
         ret['comment'] = \
-            'Failed to confirm config of repo \'{0}\': {1}'.format(name, exc)
+            'Failed to confirm config of repo {0!r}: {1}'.format(name, exc)
 
     # Clear cache of available packages, if present, since changes to the
     # repositories may change the packages that are available.
@@ -489,7 +523,7 @@ def absent(name, **kwargs):
            'changes': {},
            'result': None,
            'comment': ''}
-    repo = {}
+
     if 'ppa' in kwargs and __grains__['os'] in ('Ubuntu', 'Mint'):
         name = kwargs.pop('ppa')
         if not name.startswith('ppa:'):
@@ -554,11 +588,4 @@ def absent(name, **kwargs):
         ret['result'] = False
         ret['comment'] = 'Failed to remove repo {0}'.format(name)
 
-    # Clear cache of available packages, if present, since changes to the
-    # repositories may change the packages that are available.
-    if ret['changes']:
-        sys.modules[
-            __salt__['test.ping'].__module__
-        ].__context__.pop('pkg._avail', None)
-
     return ret
