@@ -50,6 +50,7 @@ from salt.modules import zypper
 
 # Globals
 zypper.__salt__ = dict()
+zypper.__grains__ = dict()
 zypper.__context__ = dict()
 zypper.rpm = None
 
@@ -73,7 +74,7 @@ class ZypperTestCase(TestCase):
         self.zypper_patcher_config = {
             '_get_configured_repos': Mock(side_effect=side_effect),
             '__zypper__': Mock(),
-            'get_repo': Mock()
+            'get_repo': Mock(return_value={})
         }
 
     def test_list_upgrades(self):
@@ -196,20 +197,22 @@ class ZypperTestCase(TestCase):
                          'sle-manager-tools-beta-broken-eol', 'sle-manager-tools-beta-no-eol'],
                 'vendor': 'SUSE LLC <https://www.suse.com/>',
                 'release': ['0', '0', '0', '0', '0', '0', '0'],
-                'productline': [False, False, False, False, False, False, 'sles'],
+                'productline': [None, None, None, None, None, None, 'sles'],
                 'eol_t': [None, 0, 1509408000, 1522454400, 1522454400, 1730332800, 1730332800],
                 'isbase': [False, False, False, False, False, False, True],
                 'installed': [False, False, False, False, False, False, True],
+                'registerrelease': [None, None, None, None, None, None, '123'],
             },
             'zypper-products-sle11sp3.xml': {
                 'name': ['SUSE-Manager-Server', 'SUSE-Manager-Server', 'SUSE-Manager-Server-Broken-EOL',
                          'SUSE_SLES', 'SUSE_SLES', 'SUSE_SLES', 'SUSE_SLES-SP4-migration'],
                 'vendor': 'SUSE LINUX Products GmbH, Nuernberg, Germany',
                 'release': ['1.138', '1.2', '1.2', '1.2', '1.201', '1.201', '1.4'],
-                'productline': [False, False, False, False, False, 'manager', 'manager'],
+                'productline': [None, None, None, None, None, 'manager', 'manager'],
                 'eol_t': [None, 0, 0, 0, 0, 0, 0],
                 'isbase': [False, False, False, False, False, True, True],
                 'installed': [False, False, False, False, False, True, True],
+                'registerrelease': [None, None, None, None, None, None, "42"],
             }}.items():
 
             ref_out = {
@@ -221,7 +224,7 @@ class ZypperTestCase(TestCase):
                 products = zypper.list_products()
                 self.assertEqual(len(products), 7)
                 self.assertIn(test_data['vendor'], [product['vendor'] for product in products])
-                for kwd in ['name', 'isbase', 'installed', 'release', 'productline', 'eol_t']:
+                for kwd in ['name', 'isbase', 'installed', 'release', 'productline', 'eol_t', 'registerrelease']:
                     self.assertEqual(test_data[kwd], sorted([prod.get(kwd) for prod in products]))
 
     def test_refresh_db(self):
@@ -327,6 +330,85 @@ class ZypperTestCase(TestCase):
         with patch('salt.modules.zypper.__zypper__', ZyppCallMock(return_value=get_test_data('zypper-available.txt'))):
             self.assertEqual(zypper.latest_version('vim'), '7.4.326-2.62')
 
+    @patch('salt.modules.zypper.refresh_db', MagicMock(return_value=True))
+    @patch('salt.modules.zypper._systemd_scope', MagicMock(return_value=False))
+    @patch.dict('salt.modules.zypper.__grains__', {'osrelease_info': [12, 1]})
+    def test_upgrade_success(self):
+        '''
+        Test system upgrade and dist-upgrade success.
+
+        :return:
+        '''
+        with patch('salt.modules.zypper.__zypper__.noraise.call', MagicMock()) as zypper_mock:
+            with patch('salt.modules.zypper.list_pkgs', MagicMock(side_effect=[{"vim": "1.1"}, {"vim": "1.2"}])):
+                ret = zypper.upgrade()
+                self.assertTrue(ret['result'])
+                self.assertDictEqual(ret['changes'], {"vim": {"old": "1.1", "new": "1.2"}})
+                zypper_mock.assert_any_call('update', '--auto-agree-with-licenses')
+
+            with patch('salt.modules.zypper.list_pkgs', MagicMock(side_effect=[{"vim": "1.1"}, {"vim": "1.2"}])):
+                ret = zypper.upgrade(dist_upgrade=True)
+                self.assertTrue(ret['result'])
+                self.assertDictEqual(ret['changes'], {"vim": {"old": "1.1", "new": "1.2"}})
+                zypper_mock.assert_any_call('dist-upgrade', '--auto-agree-with-licenses')
+
+            with patch('salt.modules.zypper.list_pkgs', MagicMock(side_effect=[{"vim": "1.1"}, {"vim": "1.1"}])):
+                ret = zypper.upgrade(dist_upgrade=True, dryrun=True)
+                self.assertTrue(ret['result'])
+                self.assertDictEqual(ret['changes'], {})
+                zypper_mock.assert_any_call('dist-upgrade', '--auto-agree-with-licenses', '--dry-run')
+                zypper_mock.assert_any_call('dist-upgrade', '--auto-agree-with-licenses', '--dry-run', '--debug-solver')
+
+            with patch('salt.modules.zypper.list_pkgs', MagicMock(side_effect=[{"vim": "1.1"}, {"vim": "1.1"}])):
+                ret = zypper.upgrade(dist_upgrade=True, dryrun=True, fromrepo=["Dummy", "Dummy2"], novendorchange=True)
+                self.assertTrue(ret['result'])
+                self.assertDictEqual(ret['changes'], {})
+                zypper_mock.assert_any_call('dist-upgrade', '--auto-agree-with-licenses', '--dry-run', '--from', "Dummy", '--from', 'Dummy2', '--no-allow-vendor-change')
+                zypper_mock.assert_any_call('dist-upgrade', '--auto-agree-with-licenses', '--dry-run', '--from', "Dummy", '--from', 'Dummy2', '--no-allow-vendor-change', '--debug-solver')
+
+            with patch('salt.modules.zypper.list_pkgs', MagicMock(side_effect=[{"vim": "1.1"}, {"vim": "1.2"}])):
+                ret = zypper.upgrade(dist_upgrade=True, fromrepo=["Dummy", "Dummy2"], novendorchange=True)
+                self.assertTrue(ret['result'])
+                self.assertDictEqual(ret['changes'], {"vim": {"old": "1.1", "new": "1.2"}})
+                zypper_mock.assert_any_call('dist-upgrade', '--auto-agree-with-licenses', '--from', "Dummy", '--from', 'Dummy2', '--no-allow-vendor-change')
+
+    @patch('salt.modules.zypper.refresh_db', MagicMock(return_value=True))
+    @patch('salt.modules.zypper._systemd_scope', MagicMock(return_value=False))
+    @patch.dict('salt.modules.zypper.__grains__', {'osrelease_info': [12, 1]})
+    def test_upgrade_failure(self):
+        '''
+        Test system upgrade failure.
+
+        :return:
+        '''
+        zypper_out = '''
+Loading repository data...
+Reading installed packages...
+Computing distribution upgrade...
+Use 'zypper repos' to get the list of defined repositories.
+Repository 'DUMMY' not found by its alias, number, or URI.
+'''
+
+        class FailingZypperDummy(object):
+            def __init__(self):
+                self.stdout = MagicMock(return_value=zypper_out)
+                self.stderr = MagicMock(return_value="")
+                self.exit_code = MagicMock(return_value=555)
+                self.noraise = MagicMock()
+                self.SUCCESS_EXIT_CODES = [0]
+
+            def __call__(self, *args, **kwargs):
+                return self
+
+        with patch('salt.modules.zypper.__zypper__', FailingZypperDummy()) as zypper_mock:
+            zypper_mock.noraise.call = MagicMock()
+            with patch('salt.modules.zypper.list_pkgs', MagicMock(side_effect=[{"vim": "1.1"}, {"vim": "1.1"}])):
+                ret = zypper.upgrade(dist_upgrade=True, fromrepo=["DUMMY"])
+                self.assertFalse(ret['result'])
+                self.assertEqual(ret['comment'], zypper_out.strip())
+                self.assertDictEqual(ret['changes'], {})
+                zypper_mock.noraise.call.assert_called_with('dist-upgrade', '--auto-agree-with-licenses', '--from', 'DUMMY')
+
     @patch('salt.modules.zypper.refresh_db', MagicMock(return_value=True))
     def test_upgrade_available(self):
         '''
@@ -385,6 +467,7 @@ class ZypperTestCase(TestCase):
 
         test_out = {
             'nmap': {
+                'path': u'/var/cache/zypp/packages/SLE-12-x86_64-Pool/x86_64/nmap-6.46-1.72.x86_64.rpm',
                 'repository-alias': u'SLE-12-x86_64-Pool',
                 'repository-name': u'SLE-12-x86_64-Pool'
             }
@@ -425,15 +508,22 @@ class ZypperTestCase(TestCase):
                 'stderr': ''
         }
 
-        with patch.dict(zypper.__salt__, {'cmd.run_all': MagicMock(return_value=cmd_out)}):
-            with patch.dict(zypper.__salt__, {'pkg_resource.parse_targets': MagicMock(return_value=parsed_targets)}):
-                with patch.dict(zypper.__salt__, {'pkg_resource.stringify': MagicMock()}):
-                    with patch('salt.modules.zypper.list_pkgs', ListPackages()):
-                        diff = zypper.remove(name='vim,pico')
-                        for pkg_name in ['vim', 'pico']:
-                            self.assertTrue(diff.get(pkg_name))
-                            self.assertTrue(diff[pkg_name]['old'])
-                            self.assertFalse(diff[pkg_name]['new'])
+        # If config.get starts being used elsewhere, we'll need to write a
+        # side_effect function.
+        patches = {
+            'cmd.run_all': MagicMock(return_value=cmd_out),
+            'pkg_resource.parse_targets': MagicMock(return_value=parsed_targets),
+            'pkg_resource.stringify': MagicMock(),
+            'config.get': MagicMock(return_value=True)
+        }
+
+        with patch.dict(zypper.__salt__, patches):
+            with patch('salt.modules.zypper.list_pkgs', ListPackages()):
+                diff = zypper.remove(name='vim,pico')
+                for pkg_name in ['vim', 'pico']:
+                    self.assertTrue(diff.get(pkg_name))
+                    self.assertTrue(diff[pkg_name]['old'])
+                    self.assertFalse(diff[pkg_name]['new'])
 
     def test_repo_value_info(self):
         '''
@@ -490,17 +580,8 @@ class ZypperTestCase(TestCase):
             'salt.modules.zypper', **self.zypper_patcher_config)
 
         with zypper_patcher:
-            with self.assertRaisesRegexp(
-                Exception,
-                'Specified arguments did not result in modification of repo'
-            ):
-                zypper.mod_repo(name, **{'url': url})
-            with self.assertRaisesRegexp(
-                Exception,
-                'Specified arguments did not result in modification of repo'
-            ):
-                zypper.mod_repo(name, **{'url': url, 'gpgautoimport': 'a'})
-
+            self.assertEqual(zypper.mod_repo(name, **{'url': url}),
+                             {'comment': 'Specified arguments did not result in modification of repo'})
             zypper.__zypper__.xml.call.assert_not_called()
             zypper.__zypper__.refreshable.xml.call.assert_not_called()
 
